function [alpha, beta] = createTransformAffine(A, a, varargin)
%% creates a linear transformation for general affine constraint
% This function creates vector alpha and matrix beta for the
% transformation I = alpha + beta*xc which removes a general affine 
% constraint A*I + a = 0, where A is a rectangular matrix of size (M,N) 
% with M < N;
%
% INPUTS
%  A: matrix of affine constraint, double [M x NBF]
%  a: vector of affine constraint, double [M x 1]
%  relErr (varargin): relative error of SVD cut, relatively smaller 
%                     singular values are neglected (0 < relErr < 1), 
%                     double [1 x 1]
%
% OUTPUTS
%  alpha: vector alpha from I = alpha + beta*xc, double [NBF, 1]
%  beta: matrix beta from I = alpha + beta*xc, double [NBF, NBFc]
%
% SYNTAX
%
% [alpha, beta] = createTransformAffine(A, a, varargin)
%     
% Included in AToM, info@antennatoolbox.com
% (c) 2021, Lukas Jelinek, CTU in Prague, lukas.jelinek@antennatoolbox.com
% mcode

%% relErr
if size(varargin,2) < 1
    relErr = 1e-9;
else
    relErr = varargin{1,1};
end    

%% create transformation removing affine constraint A*I + a = 0

[U1, Sigma, U2] = svd(A); % make SVD of A
NBF = size(Sigma, 2);
dSigma = diag(Sigma);

% only significant eigenvalues
ind = find(abs(dSigma) < relErr*abs(dSigma(1,1)),1);
if isempty(ind)
    ind = size(dSigma,1) + 1;
else
    disp('cutting out insignificant singular values')
end

%% define un/controllable part, U1 = [U1L, U1R], U2 = [U2L, U2R]
indU = 1:(ind - 1);
indC = ind:NBF;
U1L = U1(:,indU);
U2L = U2(:,indU);
U2R = U2(:,indC);

% figure
% plot(log10(dSigma),'-x')
% hold on
% plot(log10(dSigma(indU)),'-o')
% grid on

alpha = - U2L*diag(1./dSigma(indU,1))*(U1L')*a;
beta = U2R;

end