function [alphac, alphau, betac, betau] = createTransformPortModes(...
    Y, V, BFtoBFc, BFtoBFu)
%% creates a linear transformation for port modes
% This function creates vector alpha and matrix beta for the
% transformation corresponding to port modes of the controllable region.
% Using ralation [Ic; Iu] = [[Ycc, Ycu];[Yuc, Yuu]] * [Vc; Vu], variable
% Vc is the controllable variable xc.
%
% INPUTS
%  Y: admittance matrix, double [NBF x NBF]
%  V: excitation vector, double [NBF x 1]
%  BFtoBFc: vector of indices of controllable basis functions, 
%           integer [1 x NBFc]
%  BFtoBFu: vector of indices of uncontrollable basis functions, 
%           integer [1 x NBFu]
%
% OUTPUTS
%  alphac, alphau: controllable and uncontrollable parts of vector alpha,
%  double [NBFc x 1] and  [NBFu x 1]
%  betac, betau: controllable and uncontrollable parts of matrix beta,
%  double [NBFc x NBFc] and  [NBFu x NBFc]
%
% SYNTAX
%
% [alphac, alphau, betac, betau] = createTransformPortModes(Y, V, ...
%     BFtoBFc, BFtoBFu)
%     
% Included in AToM, info@antennatoolbox.com
% (c) 2020, Lukas Jelinek, CTU in Prague, lukas.jelinek@antennatoolbox.com
% mcode

% decomposition handles
[Mcc, Mcu, Muc, Muu, ~, Vu] = controllableRegion.decompHndls(BFtoBFc, ...
    BFtoBFu);

% vector and matrix of the transformation (separated to u,c)
alphac = Mcu(Y)*Vu(V);
alphau = Muu(Y)*Vu(V);

betac = Mcc(Y);
betau = Muc(Y);

end