function [alphac, alphau, betac, betau] = createTransformSchur(Z, V, ...
    NBFc, BFtoBFc, BFtoBFu)
%% creates a linear transformation for Schur's complement
% This function creates vector alpha and matrix beta for the
% transformation corresponding to Schur's complement of 
% [[Zcc, Zcu];[Zuc, Zuu]] * [Ic; Iu] = [Vc; Vu] with
% respect to an uncontrollable region. Variable Ic is the 
% controllable variable xc.
%
% INPUTS
%  Z: impedance matrix, double [NBF x NBF]
%  V: excitation vector, double [NBF x 1]
%  NBFc: number of basis functions in controllable region, integer [1 x 1]
%  BFtoBFc: vector of indices of controllable basis functions, 
%           integer [1 x NBFc]
%  BFtoBFu: vector of indices of uncontrollable basis functions, 
%           integer [1 x NBFu]
%
% OUTPUTS
%  alphac, alphau: controllable and uncontrollable parts of vector alpha,
%  double [NBFc x 1] and  [NBFu x 1]
%  betac, betau: controllable and uncontrollable parts of matrix beta,
%  double [NBFc x NBFc] and  [NBFu x NBFc]
%
% SYNTAX
%
% [alphac, alphau, betac, betau] = createTransformSchur(Z, V, ...
%     NBFc, BFtoBFc, BFtoBFu)
%     
% Included in AToM, info@antennatoolbox.com
% (c) 2020, Lukas Jelinek, CTU in Prague, lukas.jelinek@antennatoolbox.com
% mcode

% decomposition handles
[~, ~, Muc, Muu, ~, Vu] = controllableRegion.decompHndls(BFtoBFc, BFtoBFu);

% vector and matrix of the transformation (separated to u,c)
alphac = zeros(NBFc, 1);
alphau = mldivide(Muu(Z), Vu(V));

betac = eye(NBFc);
betau = - mldivide(Muu(Z), Muc(Z));

end