function [NBF, NBFc, NBFu, BFtoBFc, BFtoBFu] = defineControllableRegion(...
    mesh, basisFunc, userData)
%% definition of controllable and uncontrollable region
% This function defines controllable and uncontrollable region.
%
% INPUTS
%  mesh: ATOM structure containing mesh
%  basisFunc: ATOM structure containing basis functions
%  userData: strucutre with user data defining controllable and
%            uncotrollable region, struct
%
% OUTPUTS
%  NBF: total number of basis functions, integer [1 x 1]
%  NBFc: number of basis functions in controllable region, integer [1 x 1]
%  NBFu: number of basis functions in uncontrollable region,
%        integer [1 x 1]
%  BFtoBFc: vector of indices of controllable basis functions,
%           integer [1 x NBFc]
%  BFtoBFu: vector of indices of uncontrollable basis functions,
%           integer [1 x NBFu]
%
% SYNTAX
%
% [NBF, NBFc, NBFu, BFtoBFc, BFtoBFu] = defineControllableRegion(mesh, ...
%   basisFunc, userData)
%
% Included in AToM, info@antennatoolbox.com
% (c) 2020, Lukas Jelinek, CTU in Prague, lukas.jelinek@antennatoolbox.com
% mcode

if isfield(basisFunc,'nUnknowns')
    
    NBF = basisFunc.nUnknowns;
    EdgeCenters = mesh.triangleEdgeCenters(basisFunc.data(:,3),:);
    
    switch lower(userData.type)
        case 'rectprism'
            % indices of controllable region
            ind = (...
                abs(EdgeCenters(:,1) - userData.centre(1,1)) < ...
                userData.dx/2 & ...
                abs(EdgeCenters(:,2) - userData.centre(1,2)) < ...
                userData.dy/2 & ...
                abs(EdgeCenters(:,3) - userData.centre(1,3)) < ...
                userData.dz/2);
            
        case 'sphere'
            % indices of controllable region
            ind = (...
                ((EdgeCenters(:,1) - userData.centre(1,1)).^2 + ...
                (EdgeCenters(:,2) - userData.centre(1,2)).^2 + ...
                (EdgeCenters(:,3) - userData.centre(1,3)).^2) < ...
                (userData.dz/2)^2);
        otherwise
            warning(['Unknown input ', userData.type,...
                ', default is used']);
            ind = ones(NBF, 1);
    end
    
else
    
    NBF = basisFunc.nBasisFunctions;
    TetCenters = mesh.tetrahedronCentroids(basisFunc.BasisFunctionsTet, :);
    
    switch lower(userData.type)
        case 'rectprism'
            % indices of controllable region
            ind = (...
                abs(TetCenters(:,1) - userData.centre(1,1)) < ...
                userData.dx/2 & ...
                abs(TetCenters(:,2) - userData.centre(1,2)) < ...
                userData.dy/2 & ...
                abs(TetCenters(:,3) - userData.centre(1,3)) < ...
                userData.dz/2);
            
        case 'sphere'
            % indices of controllable region
            ind = (...
                ((TetCenters(:,1) - userData.centre(1,1)).^2 + ...
                (TetCenters(:,2) - userData.centre(1,2)).^2 + ...
                (TetCenters(:,3) - userData.centre(1,3)).^2) < ...
                (userData.dz/2)^2);
        otherwise
            warning(['Unknown input ', userData.type,...
                ', default is used']);
            ind = ones(NBF, 1);
    end    
end

switch lower(userData.polarity)
    case 'controllable'
        % do nothing
    case 'uncontrollable'
        ind = ~ind;
    otherwise
        warning(['Unknown input ', userData.polarity,...
            ', default is used']);
end

BFtoBFc = 1:NBF;
BFtoBFc = BFtoBFc(1,ind); % controllable indices
BFtoBFu = setdiff(1:NBF, BFtoBFc); % uncontrollable indices
NBFc = size(BFtoBFc,2);
NBFu = NBF - NBFc;

end