function [varargin] = transformQuadFormAff(alpha, beta, varargin)
%% transforms quadratic form to quadratic form
% This function transforms quadratic form to quadratic form using 
% linear transformation provided by a removal of general affine constraint.
% The transformation is defined as I = alpha + beta*xc. Quadratic 
% form I'*A*I + Re[I'*a] + a0 is transformed into 
% xc'*ATil*xc + Re[xc'*aTil] + a0Til. Matrix A is assumed Hermitian. 
%
% INPUTS
%  alpha: vector alpha from I = alpha + beta*xc, double [NBF, 1]
%  beta: matrix beta from I = alpha + beta*xc, double [NBF, NBFc]
%  varargin: triplets (A_i,a_i,a0_i) defining quadratic forms, double
%  ([N x N],[N x 1],[1 x 1])
%
% OUTPUTS
%  varargin: triplets (ATil_i,aTil_i,a0Til_i) of transformed quadratic 
% forms inside a cell, double([N x N],[N x 1],[1 x 1])
%
% SYNTAX
%
% [cell] = transformQuadFormAff(alpha, beta, A, a, a0)
%     
% Included in AToM, info@antennatoolbox.com
% (c) 2021, Lukas Jelinek, CTU in Prague, lukas.jelinek@antennatoolbox.com
% mcode

% number of quadratic forms
nForms = size(varargin,2)/3;

% transform quadratic forms one by one
for iForm = 0:(nForms - 1)
% constant term
% a0Til = real(alpha'*(A*alpha + a) + a0);
varargin{1, 3 + 3*iForm} = real(alpha'*(varargin{1, 1 + 3*iForm}*alpha + ...
    varargin{1, 2 + 3*iForm}) + varargin{1, 3 + 3*iForm});    

% linear term
% aTil = beta'*(2*A*alpha + a);
varargin{1, 2 + 3*iForm} = beta'*(2*varargin{1, 1 + 3*iForm}*alpha + ...
    varargin{1, 2 + 3*iForm});

% quadratic term
% ATil = beta'*A*beta;
varargin{1, 1 + 3*iForm} = beta'*varargin{1, 1 + 3*iForm}*beta; 

% symmetrization
% ATil = (ATil + ATil')/2;
varargin{1, 1 + 3*iForm} = (varargin{1, 1 + 3*iForm} + ...
    varargin{1, 1 + 3*iForm}')/2;

end


end