function [ATil, aTil, a0Til] = transformQuadFormCU(A, a, a0, alphac, ...
    alphau, betac, betau, BFtoBFc, BFtoBFu)
%% transforms quadratic form to quadratic form
% This function transforms quadratic form to quadratic form using 
% linear transformation provided by controllable / uncontrollable 
% separation. The transformation is defined as
% [Ic; Iu] = [alphac; alphau] + [betac; betau] * [xc; 0]. Quadratic form
% I'*A*I + Re[I'*a] + a0 is transformed into 
% xc'*ATil*xc + Re[xc'*aTil] + a0Til. Matrix A and vector a are assumed
% unsorted.
%
% INPUTS
%  A: Hermitian matrix, double [NBF x NBF]
%  a: complex vector, double [NBF x 1]
%  a0: real scalar, double [1 x 1]
%  alphac, alphau: controllable and uncontrollable parts of vector alpha,
%  double [NBFc x 1] and  [NBFu x 1]
%  betac, betau: controllable and uncontrollable parts of matrix beta,
%  double [NBFc x NBFc] and [NBFu x NBFc]
%  BFtoBFc: vector of indices of controllable basis functions, 
%           integer [1 x NBFc]
%  BFtoBFu: vector of indices of uncontrollable basis functions, 
%           integer [1 x NBFu]
%
% OUTPUTS
%  ATil: Hermitian matrix, double [NBF x NBF]
%  aTil: complex vector, double [NBF x 1]
%  a0Til: real scalar, double [1 x 1]
%
% SYNTAX
%
% [ATil, aTil, a0Til] = transformQuadFormCU(A, a, a0, alphac, ...
%     alphau, betac, betau, BFtoBFc, BFtoBFu)
%     
% Included in AToM, info@antennatoolbox.com
% (c) 2020, Lukas Jelinek, CTU in Prague, lukas.jelinek@antennatoolbox.com
% mcode

% decomposition handles
[Mcc, Mcu, Muc, Muu, Vc, Vu] = controllableRegion.decompHndls(BFtoBFc, ...
    BFtoBFu);

% quadratic term
ATil = betac'*(Mcc(A)*betac + Mcu(A)*betau) + ...
       betau'*(Muc(A)*betac + Muu(A)*betau);

% symmetrization
ATil = (ATil + ATil')/2;

% linear term
aTil = betac'*(2*(Mcc(A)*alphac + Mcu(A)*alphau) + Vc(a)) + ...
        betau'*(2*(Muc(A)*alphac + Muu(A)*alphau) + Vu(a));

% constant term
a0Til = real(alphac'*((Mcc(A)*alphac + Mcu(A)*alphau) + Vc(a)) + ...
        alphau'*((Muc(A)*alphac + Muu(A)*alphau) + Vu(a)) + ...
        a0);

end