function [PaOpt, PsOpt, xOpt, lambdaVec, tocTime] = ...
    AbsorbScattParetoConstraintZpowerConst (OP, V, varargin)
%% evaluates Pareto optimal set between absorption and scattering
% This function evaluates Pareto optimal set between absorbption and 
% scattering with a constraint on complex power conservation. 
% Mathematically this function solves the following problem
% min over x { - x'*OP.Rmat*x },
% s.t. { x'*(OP.R0 + OP.Rmat)*x + Re[x'*(-V)] = 0 }
% s.t. { x'*(OP.X0 + OP.Xmat)*x + Re[x'*(1i*V)] = 0 }
% s.t. { 0.5*x'*OP.R0*x = OP.alpha*OP.PsMax }
%
% INPUTS
%  OP.alpha: parameter of the Pareto sweep, double [1 x 1]
%  OP.PsMax: Maximum attainable value of scattered power, double [1 x 1]
%  OP.R0: P_rad = 0.5*x'*OP.R0*x, double [N x N]
%  OP.Rmat: P_lost = 0.5*x'*OP.Rmat*x, double [N x N]
%  OP.X0: P_react = 0.5*x'*(OP.X0 + OP.Xmat)*x, double [N x N]
%  OP.Xmat: P_react = 0.5*x'*(OP.X0 + OP.Xmat)*x, double [N x N]
%  V: (OP.R0 + OP.Rmat + 1i*OP.X0 + 1i*OP.Xmat)*x = V, double [N x 1]
%  varargin: property-value pairs:
%  'lambdaVec', initial values of Lagrange multipliers, double [nConst x 1]
%  'verbosity', {0, 1}, 0 --> do not report, > 1 report 
%
% OUTPUTS
%  PaOpt: optimal absorbed power, double [1 x 1]
%  PsOpt: optimal scattered power, double [1 x 1]
%  xOpt: solution to the problem, double [N x 1]
%  lambdaVec: optimal value of Lagrange's multiplier, double [nConst x 1]
%  tocTime: duration of evaluation
%
% SYNTAX
%
% [PaOpt, PsOpt, xOpt, lambdaVec, tocTime] = ... 
% AbsorbScattParetoConstraintZpowerConst(OP, V)
% 
% [PaOpt, PsOpt, xOpt, lambdaVec, tocTime] = ...
% AbsorbScattParetoConstraintZpowerConst(OP, V, 'lambdaVec', lambdaVec)
% 
% [PaOpt, PsOpt, xOpt, lambdaVec, tocTime] = ...
% AbsorbScattParetoConstraintZpowerConst(OP, V, 'lambdaVec', lambdaVec, ...
% 'verbosity', 1)
%
% Included in AToM, info@antennatoolbox.com
% (c) 2020, Lukas Jelinek, CTU in Prague, lukas.jelinek@antennatoolbox.com
% mcode

%% initialization
lambdaVec = rand(3,1); % Initial values of Lagrange multipliers
verbosity = 0;

% input parsing
if size(varargin,2) >= 2
   for inputInd = 1:2:(size(varargin,2) - 1)
      switch lower(varargin{1,inputInd})
         case 'lambdavec'
            lambdaVec = varargin{1, inputInd + 1};
         case 'verbosity'
            verbosity = varargin{1, inputInd + 1};
         otherwise
            warning(['Unknown input ', varargin{1,inputInd},...
               ', default is used']);
      end
   end
end

tic
%% Optimization

[primalOpt, gOpt, xOpt, ~, lambdaVec] = solvers.QCQP.QNCQPQuadLin ...
   (OP.sign*OP.Rmat, 0*V, 0, lambdaVec, verbosity, ...
    OP.R0 + OP.Rmat, - V, 0, ...
    OP.X0 + OP.Xmat, 1i*V, 0, ...
    0.5*OP.R0, 0*V, - OP.alpha*OP.PsMax );

% absorbed power
PaOpt = 0.5*(xOpt'*OP.Rmat*xOpt); 

% scattered power
PsOpt = 0.5*(xOpt'*OP.R0*xOpt); 

%% check for error (convergence or dual gap)
% all outputs should be the same
allOutputs = repmat([gOpt, primalOpt, ...
    OP.sign*2*real(PaOpt)], [3, 1]);

% relative error (all vs all)
err = 2* max(max(abs((allOutputs - allOutputs.')./ ...
                     (allOutputs + allOutputs.') )));
if err > 1e-4
   warning('Dual gap or failed convergence')
end

tocTime = toc;
end