function [G, Ivec, lambdaVec, tocTime] = maxGvsDelta(OP, delta, varargin)
%% minGvsDelta - Optimal gain with power constraints and delta
% This function minimizes gain constrained by power dissipation constraint and
% delta. Mathematicaly this function solves the following problem
% min { x'*(-OP.U)*x },
% s.t. { x'*(OP.R0 + OP.Rmat)*x + (-1) = 0 },
%      { (1 + delta)*x'*OP.R0*x + (-1) = 0 }.
%
%  INPUTS
%   OP: structure of matrix operators R0, Rmat, U, X0
%   varargin: 'lambdaVec', initial Lagrange's multipiers,
%     'verbosity', reporting flag (0 -> do not report, 1 -> report,
%        2 -> detailed report, 3 -> widely detailed report)
%
%  OUTPUTS
%   G: optimal gain
%   Ivec: argmin vector
%   lambdaVec: optimal value of Lagrange's multipliers
%   tocTime: duration of evaluation
%
%  SYNTAX
%
%   [Gopt, Ivec, lambdaVec, tocTime] = maxGvsDela(OP, delta)
%   [Gopt, Ivec, lambdaVec, tocTime] = maxGvsDelta(OP, delta, varargin)
%
% AToM package Fundamental Bounds, info@antennatoolbox.com
% (c) 2020, Jakub Liska, CTU in Prague, jakub.liska@antennatoolbox.com
% mcode

% initialization
lambdaVec = zeros(1,2);
verbosity = 0;

if size(varargin,2) >= 2
   for inputInd = 1:2:size(varargin,2) - 1
      switch varargin{1,inputInd}
         case 'lambdaVec'
            lambdaVec = reshape(varargin{1, inputInd + 1}, 1, 2);
         case 'verbosity'
            verbosity = varargin{1, inputInd + 1};
         otherwise
            warning(['Unknown input ', varargin{1,inputInd},...
               ', default values used!']);
      end
   end
else
   disp('Default values used.');
end

% evaluation
t0=tic;
[~, ~, Ivec, lambdaVec] = solvers.QCQP.minAstBn( - OP.U,...
   lambdaVec, verbosity,...
   OP.R0 + OP.Rmat, {}, - 1,...
   OP.R0 - OP.Rmat/delta, {}, 0);
tocTime = toc(t0);

G = 8*pi*real((Ivec'*OP.U*Ivec)/(Ivec'*(OP.Rmat + OP.R0)*Ivec)); % optimal value
end