function [FOpt, xOpt, lambdaVec, tocTime] = maxPurcellCUZpowerConst ...
    (OP, V, NBFc, NBF, BFtoBFc, BFtoBFu, varargin)
%% maxPurcellCUZpowerConst evaluates upper bound to Purcell's factor
% This function evaluates upper bound to Purcell's factor of an 
% uncontrollable region with a constraint on complex power conservation. The 
% degrees of % freedom are only within the controllable region. Mathematically 
% this function solves the following problem
% min over xc { - x'*OP.R0*x },
% s.t. { x'*(OP.R0 + OP.Rmat)*x + Re[x'*(-V)] = 0 }
% s.t. { x'*(OP.X0 + OP.Xmat)*x + Re[x'*(1i*V)] = 0 }
% s.t. { x = alpha + beta*xc }
% where the last constraint defines the controllable and uncontrollable
% region.
%
% INPUTS
%  OP.R0: P_rad = 0.5*x'*OP.R0*x, double [N x N]
%  OP.Rmat: P_lost = 0.5*x'*OP.Rmat*x, double [N x N]
%  OP.X0: P_react = 0.5*x'*(OP.X0 + OP.Xmat)*x, double [N x N]
%  OP.Xmat: P_react = 0.5*x'*(OP.X0 + OP.Xmat)*x, double [N x N]
%  V: (OP.R0 + OP.Rmat + 1i*OP.X0 + 1i*OP.Xmat)*x = V, double [N x 1]
%  NBFc: number of basis functions in controllable region, integer [1 x 1]
%  NBF: total number of basis functions, integer [1 x 1]
%  BFtoBFc: vector of indices of controllable basis functions, 
%           integer [1 x NBFc]
%  BFtoBFu: vector of indices of uncontrollable basis functions, 
%           integer [1 x NBFu]
%  varargin: property-value pairs:
%  'lambdaVec', initial values of Lagrange multipliers, double [nConst x 1]
%  'verbosity', {0, 1}, 0 --> do not report, > 1 report 
%
% OUTPUTS
%  FOpt: optimal Purcell's factor, double [1 x 1]
%  xOpt: solution to the problem, double [N x 1]
%  lambdaVec: optimal value of Lagrange's multiplier, double [nConst x 1]
%  tocTime: duration of evaluation
%
% SYNTAX
%
% [FOpt, xOpt, lambdaVec, tocTime] = maxPurcellCUZpowerConst ...
%     (OP, V, NBFc, NBF, BFtoBFc, BFtoBFu)
% 
% [FOpt, xOpt, lambdaVec, tocTime] = maxPurcellCUZpowerConst ...
%     (OP, V, NBFc, NBF, BFtoBFc, BFtoBFu, 'lambdaVec', lambdaVec)
% 
% [FOpt, xOpt, lambdaVec, tocTime] = maxPurcellCUZpowerConst ...
%     (OP, V, NBFc, NBF, BFtoBFc, BFtoBFu, ...
%      'lambdaVec', lambdaVec, 'verbosity', 1)
%
% Included in AToM, info@antennatoolbox.com
% (c) 2020, Lukas Jelinek, CTU in Prague, lukas.jelinek@antennatoolbox.com
% mcode

%% initialization
lambdaVec = [-1; 0]; % Initial values of Lagrange multipliers
verbosity = 0;

% input parsing
if size(varargin,2) >= 2
   for inputInd = 1:2:(size(varargin,2) - 1)
      switch lower(varargin{1,inputInd})
         case 'lambdavec'
            lambdaVec = varargin{1, inputInd + 1};
         case 'verbosity'
            verbosity = varargin{1, inputInd + 1};
         otherwise
            warning(['Unknown input ', varargin{1,inputInd},...
               ', default is used']);
      end
   end
end

tic

%% create transformation to controllabe region
[alphac, alphau, betac, betau] = controllableRegion.createTransformSchur( ...
    OP.R0 + OP.Rmat + 1i*OP.X0 + 1i*OP.Xmat, V, NBFc, BFtoBFc, BFtoBFu);

%% define tranformed operators

[ATil, aTil, a0Til] = controllableRegion.transformQuadFormCU(...
    - OP.R0, 0*V, 0, ...
    alphac, alphau, betac, betau, BFtoBFc, BFtoBFu);

[BTil, bTil, b0Til] = controllableRegion.transformQuadFormCU(...
    OP.R0 + OP.Rmat, - V, 0, ...
    alphac, alphau, betac, betau, BFtoBFc, BFtoBFu);

[CTil, cTil, c0Til] = controllableRegion.transformQuadFormCU(...
    OP.X0 + OP.Xmat, 1i*V, 0, ...
    alphac, alphau, betac, betau, BFtoBFc, BFtoBFu);

%% Optimization
[primalOpt, gOpt, xc, ~, lambdaVec] = solvers.QCQP.QNCQPQuadLin ...
   (ATil, aTil, a0Til, lambdaVec, verbosity, ...
    BTil, bTil, b0Til, ...
    CTil, cTil, c0Til);

%% Results

% optimal current density on the entire structure
xOpt = controllableRegion.assemblyCurrent(xc, alphac, alphau, betac, ...
    betau, NBF, BFtoBFc, BFtoBFu);

% decomposition handles 
[~, ~, ~, Muu, ~, Vu] = controllableRegion.decompHndls (BFtoBFc, BFtoBFu);

% current in the uncontrollable region without the controllable region
I0 = mldivide(Muu(OP.R0 + OP.Rmat + 1i*OP.X0 + 1i*OP.Xmat), Vu(V));

% optimal Purcell's factor
PradTot = 0.5*(xOpt'*OP.R0*xOpt);
FOpt = PradTot / (0.5*I0'*Muu(OP.R0)*I0);

%% check for error (convergence or dual gap)
% all outputs should be the same
allOutputs = repmat([gOpt, primalOpt, - 2*real(PradTot)], [3, 1]);

% relative error (all vs all)
err = 2* max(max(abs((allOutputs - allOutputs.')./ ...
                     (allOutputs + allOutputs.') )));
if err > 1e-6
   disp('Dual gap or failed convergence')
end

tocTime = toc;
end