function [PaOpt, PsOpt, PeOpt, xOpt, lambdaVec, tocTime] = ...
    minMaxPaPsPeZpowerConst(OP, V, minMax, whatPower, varargin)
%% minMaxPaPsPeZpowerConst evaluates lower bound to Pa,Ps,Pe
% This function evaluates lower bound to absorption, scattering or
% extinction with a constraint on complex power conservation and with or
% without an arbitrary affine constraint. Mathematically this function
% solves the following problem
% min/max over xc { x'*(OP.Rmat)*x }
% min/max over xc { x'*(OP.R0)*x }
% min/max over xc { x'*(OP.R0 + OP.Rmat)*x }
% s.t. { x'*(OP.R0 + OP.Rmat)*x + Re[x'*(-V)] = 0 }
% s.t. { x'*(OP.X0 + OP.Xmat)*x + Re[x'*(1i*V)] = 0 }
% s.t. { x = alpha + beta*xc }
% where the last constraint defines transformation removing a general
% affine constraint OP.Aaff*I + OP.aAff = 0. Use OP.Aaff = [] and
% OP.aAff = [] if there is no affine constraint.
%
% INPUTS
%  OP.R0: P_rad = 0.5*x'*OP.R0*x, double [N x N]
%  OP.Rmat: P_lost = 0.5*x'*OP.Rmat*x, double [N x N]
%  OP.X0: P_react = 0.5*x'*(OP.X0 + OP.Xmat)*x, double [N x N]
%  OP.Xmat: P_react = 0.5*x'*(OP.X0 + OP.Xmat)*x, double [N x N]
%  V: (OP.R0 + OP.Rmat + 1i*OP.X0 + 1i*OP.Xmat)*x = V, double [N x 1]
%  OP.Aaff: matrix of the affine constraint, double [M x N]
%  OP.aAff: vector of the affine constraint, double [M x N]
%  minMax: flag denoting if minimization or maximization is to be made,
%          'min' or 'max'
%  whatPower: flag denoting which power is to be optimized,
%             'Pa' or 'Ps' or 'Pe'
%  varargin: property-value pairs:
%  'lambdaVec', initial values of Lagrange multipliers, double [nConst x 1]
%  'verbosity', {0, 1}, 0 --> do not report, > 1 report
%
% OUTPUTS
%  PaOpt: optimal absorbed power, double [1 x 1]
%  PsOpt: optimal scattered power, double [1 x 1]
%  PeOpt: optimal extincted power, double [1 x 1]
%  xOpt: solution to the problem, double [N x 1]
%  lambdaVec: optimal value of Lagrange's multiplier, double [nConst x 1]
%  tocTime: duration of evaluation
%
% SYNTAX
%
% [PaOpt,PsOpt,PeOpt, xOpt, lambdaVec, tocTime] = ...
%     minMaxPaPsPeZpowerConst(OP, V, minMax, whatPower)
%
% [PaOpt,PsOpt,PeOpt, xOpt, lambdaVec, tocTime] = ...
%     minMaxPaPsPeZpowerConst(OP, V, minMax, whatPower, 'lambdaVec', ...
%     lambdaVec)
%
% [PaOpt,PsOpt,PeOpt, xOpt, lambdaVec, tocTime] = ...
%     minMaxPaPsPeZpowerConst(OP, V, minMax, whatPower, ...
%      'lambdaVec', lambdaVec, 'verbosity', 1)
%
% Included in AToM, info@antennatoolbox.com
% (c) 2021, Lukas Jelinek, CTU in Prague, lukas.jelinek@antennatoolbox.com
% mcode

%% initialization
lambdaVec = [0; 1]; % Initial values of Lagrange multipliers
verbosity = 0;

% input parsing
if size(varargin,2) >= 2
    for inputInd = 1:2:(size(varargin,2) - 1)
        switch lower(varargin{1,inputInd})
            case 'lambdavec'
                lambdaVec = varargin{1, inputInd + 1};
            case 'verbosity'
                verbosity = varargin{1, inputInd + 1};
            otherwise
                warning(['Unknown input ', varargin{1,inputInd},...
                    ', default is used']);
        end
    end
end

tic

% decide on min/max
switch lower(minMax)
    case 'max'
        multi = -1;
        disp('maximization is made');
    otherwise
        multi = 1;
        disp('minimization is made');
end

if ~(isempty(OP.Aaff) && isempty(OP.aAff))
    %% define transformation removing the affine constraint
    [alpha, beta] = controllableRegion.createTransformAffine(...
        OP.Aaff, OP.aAff);
    
    %% define tranformed operators
    % decide and solve the corresponding optimal power
    switch lower(whatPower)
        case 'pa'
            disp('absorption is optimized');
            [aCell] = ...
                controllableRegion.transformQuadFormAff(alpha, beta, ...
                multi*OP.Rmat, 0*V, 0);
        case 'ps'
            disp('scattering is optimized');
            [aCell] = ...
                controllableRegion.transformQuadFormAff(alpha, beta, ...
                multi*OP.R0, 0*V, 0);
        case 'pe'
            disp('extinction is optimized');
            [aCell] = ...
                controllableRegion.transformQuadFormAff(alpha, beta, ...
                multi*(OP.R0 + OP.Rmat), 0*V, 0);
        otherwise
            disp('extinction is optimized');
            [aCell] = ...
                controllableRegion.transformQuadFormAff(alpha, beta, ...
                multi*(OP.R0 + OP.Rmat), 0*V, 0);
    end
    
    [bcCell] = ... 
        controllableRegion.transformQuadFormAff(alpha, beta, ...
        OP.R0 + OP.Rmat, - V, 0, ...
        OP.X0 + OP.Xmat, 1i*V, 0);
    
else
    % decide and solve the corresponding optimal power
    aCell = cell(1,3);
    switch lower(whatPower)
        case 'pa'
            disp('absorption is optimized');
            aCell{1,1} = multi*OP.Rmat;
            aCell{1,2} = 0*V;
            aCell{1,3} = 0;
        case 'ps'
            disp('scattering is optimized');
            aCell{1,1} = multi*OP.R0;
            aCell{1,2} = 0*V;
            aCell{1,3} = 0;
        case 'pe'
            disp('extinction is optimized');
            aCell{1,1} = multi*(OP.Rmat + OP.R0);
            aCell{1,2} = 0*V;
            aCell{1,3} = 0;
        otherwise
            disp('extinction is optimized');
            aCell{1,1} = multi*OP.Rmat;
            aCell{1,2} = 0*V;
            aCell{1,3} = 0;
    end
    
    bcCell = cell(1,6);
    bcCell{1,1} = OP.R0 + OP.Rmat;
    bcCell{1,2} = - V;
    bcCell{1,3} = 0;
    
    bcCell{1,4} = OP.X0 + OP.Xmat;
    bcCell{1,5} = 1i*V;
    bcCell{1,6} = 0;
    
    alpha = zeros(size(bcCell{1,4},1),1);
    beta = eye(size(bcCell{1,4},1));
    
end
%% Optimization
[primalOpt, gOpt, xc, ~, lambdaVec] = solvers.QCQP.QNCQPQuadLin ...
    (aCell{1,1:3}, lambdaVec, verbosity, ...
    bcCell{1,1:6});

%% Results
% optimal current density on the entire structure
xOpt = alpha + beta*xc;

% power absorbed, scattered and extincted by the entire structure
PaOpt = 0.5*(xOpt'*(OP.Rmat)*xOpt);
PsOpt = 0.5*(xOpt'*(OP.R0)*xOpt);
PeOpt = 0.5*(xOpt'*(OP.R0 + OP.Rmat)*xOpt);

%% check for error (convergence or dual gap)
% all outputs should be the same
switch lower(whatPower) % decide and solve the corresponding optimal power
    case 'pa'
        allOutputs = repmat([gOpt, primalOpt, multi*2*real(PaOpt)], [3, 1]);
    case 'ps'
        allOutputs = repmat([gOpt, primalOpt, multi*2*real(PsOpt)], [3, 1]);
    case 'pe'
        allOutputs = repmat([gOpt, primalOpt, multi*2*real(PeOpt)], [3, 1]);
    otherwise
        allOutputs = repmat([gOpt, primalOpt, multi*2*real(PaOpt)], [3, 1]);
end

% relative error (all vs all)
err = 2* max(max(abs((allOutputs - allOutputs.')./ ...
    (allOutputs + allOutputs.') )));
if err > 1e-6
    disp('Dual gap or failed convergence')
end

tocTime = toc;
end