function [Qopt, Ivec, lambdaVec, tocTime] = minQselfRes(OP, varargin)
%% minQselfrRes - Optimal Q with power constraints
% This function minimizes Q-factor constrained by complex power conservation and
% selfresonance. Mathematicaly this function solves the following problem
% min { x'*(-OP.R0)*x },
% s.t. { x'*OP.omW*x + (-1) = 0 },
%      { x'*(OP.X0 + OP.Xmat)*x = 0 }.
%
%  INPUTS
%   OP: structure of matrix operators R0, omW, X0
%   varargin: 'lambdaVec', initial Lagrange's multipiers,
%     'verbosity', reporting flag (0 -> do not report, 1 -> report,
%        2 -> detailed report, 3 -> widely detailed report)
%
%  OUTPUTS
%   Qopt: minimum of primar function
%   Ivec: argmin vector
%   lambdaVec: optimal value of Lagrange's multipliers
%   tocTime: duration of evaluation
%
%  SYNTAX
%
%   [Qopt, Ivec, lambdaVec, tocTime] = minQselfRes(OP)
%   [Qopt, Ivec, lambdaVec, tocTime] = minQselfRes(OP, varargin)
%
% AToM package Fundamental Bounds, info@antennatoolbox.com
% (c) 2021, Jakub Liska, CTU in Prague, jakub.liska@antennatoolbox.com
% mcode

% initialization
lambdaVec = zeros(1,2);
verbosity = 0;

if size(varargin,2) >= 2
   for inputInd = 1:2:size(varargin,2) - 1
      switch varargin{1,inputInd}
         case 'lambdaVec'
            lambdaVec = reshape(varargin{1, inputInd + 1}, 1, 2);
         case 'verbosity'
            verbosity = varargin{1, inputInd + 1};
         otherwise
            warning(['Unknown input ', varargin{1,inputInd},...
               ', default values used!']);
      end
   end
else
   disp('Default values used.');
end

% evaluation
t0=tic;
[~, triplets] = solvers.normalizeTriplet(- OP.R0, [], 0,...
    OP.omW, [], 0,...
    OP.X0 + OP.Xmat, [], 0);
[~, ~, Ivec, lambdaVec] = solvers.QCQP.minAstBn( triplets{1},...
   lambdaVec, verbosity,...
   triplets{4}, {}, - 1,...
   triplets{7:end});
tocTime = toc(t0);

Qopt = 0.5*real(((Ivec'*OP.omW*Ivec) + abs(Ivec'*OP.X0*Ivec))/...
   (Ivec'*OP.R0*Ivec)); % optimal value
end