function [Q, Ivec, lambdaVec, tocTime] = minQselfResVsDelta(...
   OP, delta, varargin)
%% minQselfResVsDelta - Optimal Q with power constraints vs. efficiency
% This function minimizes Q-factor constrained by complex power conservation,
% selfresonance and efficiency. Mathematicaly this function solves the following
% problem
% min { -x'*OP.R0*x },
% s.t. { x'*OP.omW*x + (-1) = 0 },
%      { x'*OP.X0*x = 0 },
%      { x'*(OP.Rmat/delta - OP.R0)*x = 0}.
%
%  INPUTS
%   OP: structure of matrix operators R0, omW, X0, Rmat
%   delta: required efficiency
%   varargin: 'lambdaVec', initial Lagrange's multipiers,
%     'verbosity', reporting flag (0 -> do not report, 1 -> report,
%        2 -> detailed report, 3 -> widely detailed report)
%
%  OUTPUTS
%   Q: optimal Q-factor
%   Ivec: argmin vector
%   lambdaVec: optimal value of Lagrange's multipliers
%   tocTime: duration of evaluation
%
%  SYNTAX
%
%   [Q, Ivec, lambdaVec, tocTime] = minQselfResVsDelta(OP)
%   [Q, Ivec, lambdaVec, tocTime] = minQselfResVsDelta(OP, varargin)
%
% AToM package Fundamental Bounds, info@antennatoolbox.com
% (c) 2021, Jakub Liska, CTU in Prague, jakub.liska@antennatoolbox.com
% mcode

% initialization
lambdaVec = zeros(1,3);
verbosity = 0;

if size(varargin,2) >= 2
   for inputInd = 1:2:size(varargin,2) - 1
      switch varargin{1,inputInd}
         case 'lambdaVec'
            lambdaVec = reshape(varargin{1, inputInd + 1}, 1, 3);
         case 'verbosity'
            verbosity = varargin{1, inputInd + 1};
         otherwise
            warning(['Unknown input ', varargin{1,inputInd},...
               ', default values used!']);
      end
   end
else
   disp('Default values used.');
end

[~, triplets] = solvers.normalizeTriplet( OP.R0, [], 0,...
    OP.omW, [], 0,...
    OP.X0 + OP.Xmat, [], 0,...
    OP.Rmat/delta - OP.R0, [], 0);

% evaluation
t0=tic;
[~, ~, Ivec, lambdaVec] = solvers.QCQP.minAstBn( - triplets{1},...
   lambdaVec, verbosity,...
   triplets{4}, [], - 1,...
   triplets{7:end});
tocTime = toc(t0);

Q = 0.5*real(((Ivec'*OP.omW*Ivec) + abs(Ivec'*OP.X0*Ivec))/(Ivec'*OP.R0*Ivec));
end