function [Q, Ivec, lambdaVec, tocTime] = minQselfResVsG(OP, G, varargin)
%% minQselfResVsG - Optimal Q with power constraints vs. gain
% This function minimizes Q-factor constrained by complex power conservation,
% selfresonance and gain. Mathematicaly this function solves the following
% problem
% min { 0.5*x'*OP.omW*x },
% s.t. { x'*OP.Rmat*x + (-1) = 0 },
%      { x'*OP.X0*x = 0 },
%      { x'*(OP.R0 - OP.U/G)*x + 1 = 0}.
%
%  INPUTS
%   OP: structure of matrix operators R0, omW, X0, Rmat, U
%   G: required gain
%   varargin: 'lambdaVec', initial Lagrange's multipiers,
%     'verbosity', reporting flag (0 -> do not report, 1 -> report,
%        2 -> detailed report, 3 -> widely detailed report)
%
%  OUTPUTS
%   Q: minimum of primar function
%   Ivec: argmin vector
%   lambdaVec: optimal value of Lagrange's multipliers
%   tocTime: duration of evaluation
%
%  SYNTAX
%
%   [Q, Ivec, lambdaVec, tocTime] = minQselfResVsG(OP)
%   [Q, Ivec, lambdaVec, tocTime] = minQselfResVsG(OP, varargin)
%
% AToM package Fundamental Bounds, info@antennatoolbox.com
% (c) 2020, Jakub Liska, CTU in Prague, jakub.liska@antennatoolbox.com
% mcode

% initialization
lambdaVec = zeros(1,3);
verbosity = 0;

if size(varargin,2) >= 2
   for inputInd = 1:2:size(varargin,2) - 1
      switch varargin{1,inputInd}
         case 'lambdaVec'
            lambdaVec = reshape(varargin{1, inputInd + 1}, 1, 3);
         case 'verbosity'
            verbosity = varargin{1, inputInd + 1};
         otherwise
            warning(['Unknown input ', varargin{1,inputInd},...
               ', default values used!']);
      end
   end
else
   disp('Default values used.');
end

% evaluation
t0=tic;
[~, ~, Ivec, lambdaVec] = solvers.QCQP.minAstBn( -OP.R0,...
   lambdaVec, verbosity,...
   OP.omW, {}, - 1,...
   OP.X0 + OP.Xmat, {}, 0,...
   OP.Rmat + OP.R0 - 8*pi*OP.U/G, {}, 0);
tocTime = toc(t0);

Q = 0.5*real(((Ivec'*OP.omW*Ivec) + abs(Ivec'*OP.X0*Ivec))/(Ivec'*OP.R0*Ivec));
end