function [QOpt, Qext, xOpt, lambdaVec, tocTime] = minQselfResZpowerConst ...
   (OP, V, varargin)
%% minQselfResZpowerConst Optimal Q with power constraints
% This function minimizes Q-factor with a constraint on complex power 
% conservation. Mathematicaly this function solves the following problem
% min { x'*(- OP.R0)*x },
% s.t. { x'*OP.omW*x + (-1) = 0 }
% s.t. { x'*(OP.X0 + OP.Xmat)*x = 0 }
% s.t. { x'*(OP.R0 + OP.Rmat)*x + Re[x'*(-V)] = 0 }
% s.t. { x'*(OP.X0 + OP.Xmat)*x + Re[x'*(1i*V)] = 0 }
%
% INPUTS
%  OP.R0: P_rad = 0.5*x'*OP.R0*x, double [N x N]
%  OP.Rmat: P_lost = 0.5*x'*OP.Rmat*x, double [N x N]
%  OP.X0: P_react = 0.5*x'*(OP.X0 + OP.Xmat)*x, double [N x N]
%  OP.Xmat: P_react = 0.5*x'*(OP.X0 + OP.Xmat)*x, double [N x N]
%  OP.omW: W_sto = 0.5*x'*OP.omW*x, double [N x N]
%  V: (OP.R0 + OP.Rmat + 1i*OP.X0 + 1i*OP.Xmat)*x = V, double [N x 1]
%  varargin: property-value pairs:
%  'lambdaVec', initial values of Lagrange multipliers, double [nConst x 1]
%  'verbosity', {0, 1}, 0 --> do not report, > 1 report 
%
% OUTPUTS
%  QOpt: minimum of dual function, double [1 x 1]
%  Qext: external tuning, , double [1 x 1]
%  xOpt: solution to the problem, double [N x 1]
%  lambdaVec: optimal value of Lagrange's multiplier, double [nConst x 1]
%  tocTime: duration of evaluation
%
% SYNTAX
%
% [QOpt, Qext, xOpt, lambdaVec, tocTime] = optimProblems.minQpowerConst ...
%    (OP, V)
% 
% [QOpt, Qext, xOpt, lambdaVec, tocTime] = optimProblems.minQpowerConst ...
%    (OP, V,'lambdaVec',lambdaVec)
% 
% [QOpt, Qext, xOpt, lambdaVec, tocTime] = optimProblems.minQpowerConst ...
%    (OP, V, 'lambdaVec', lambdaVec, 'verbosity', 1)
% 
%
% Included in AToM, info@antennatoolbox.com
% (c) 2019, Lukas Jelinek, CTU in Prague, lukas.jelinek@antennatoolbox.com
% mcode

%% initialization
lambdaVec = rand(4,1); % Initial values of Lagrange multipliers
verbosity = 0;

% input parsing
if size(varargin,2) >= 2
   for inputInd = 1:2:(size(varargin,2) - 1)
      switch lower(varargin{1,inputInd})
         case 'lambdavec'
            lambdaVec = varargin{1, inputInd + 1};
         case 'verbosity'
            verbosity = varargin{1, inputInd + 1};
         otherwise
            warning(['Unknown input ', varargin{1,inputInd},...
               ', default is used']);
      end
   end
end

% size of the problem
N = size(OP.omW,1);

tic
% optimization
[primalOpt, gOpt, xOpt, ~, lambdaVec] = solvers.QCQP.QNCQPQuadLin (...
  - OP.R0, zeros(N,1), 0, ...
   lambdaVec, verbosity, ...
   OP.omW, zeros(N,1), -1, ...
   OP.X0 + OP.Xmat, zeros(N,1), 0, ...
   OP.R0 + OP.Rmat, -V, 0, ...   
   OP.X0 + OP.Xmat, 1i*V, 0);

% results
Prad = 0.5*((xOpt')*OP.R0*xOpt); % radiated power
Qext = 0.25*abs((xOpt')*(OP.X0 + OP.Xmat)*xOpt) / Prad; % tuning part of Q
QOpt = 0.25*((xOpt')* OP.omW * xOpt) / Prad + Qext; % Q-factor

%% check for error (convergence or dual gap)

% all outputs should be the same
allOutputs = repmat([gOpt, primalOpt, real(xOpt'*(-OP.R0)*xOpt), ...
                   real(-1/(2*QOpt))], [4, 1]);

% relative error (all vs all)
err = 2* max(max(abs( (allOutputs - allOutputs.')./ ...
                      (allOutputs + allOutputs.') )));

if err > 1e-6
   disp('Dual gap or failed convergence')
end

%%

tocTime = toc;

end