function [primalOpt, gOpt, xOpt, constVal, lambdaVec] = QNCQPQuadLin ...
    (A, a, a0, lambdaVec, verbosity, varargin)
%% quadratic program with N quadratic constraints
% This function solves quadratic program with N quadratic constraints
% of the form
% min { x'*A*x + Re[x'*a] + a0 },
% s.t. { x'*B_i*x + Re[x'*b_i] + b0_i = 0 }, i = 1..nConst.
% It is assumed that A, B_i are hermitian and matrix B_1 is full rank and
% positive definite. It is further assumed that at least one of {a, b_i} is
% non-zero.
%
% INPUTS
%  A: matrix of the optimized metric, double [N x N]
%  a: vector of the optimized metric, double [N x 1]
%  a0: real number of the optimized metric, double [1 x 1]
%  lambdaVec: initial guess of Lagrange's multipliers, real [nConst x 1]
%  verbosity: reporting flag (0 -> do not report, 1 -> basic report
%     2 -> basic report + PSD boundary proximity (slowdown), 
%     3 -> full report including graphics (slow)), integer [1 x 1]
%  varargin: triplets (B_i,b_i,b0_i) of the constraint, double
%  ([N x N],[N x 1],[1 x 1])
%
% OUTPUTS
%  primalOpt: value of optimized metric at xOpt, double [1 x 1]
%  gOpt: minimum of dual function, double [1 x 1]
%  xOpt: solution to the problem, double [N x 1]
%  constVal: constraint value at xOpt, double [nConst x 1]
%  lambdaVec: optimal value of Lagrange's multiplier, double [nConst x 1]
%
% SYNTAX
%
% [primalOpt, gOpt, xOpt, constVal, lambdaVec] = solvers.QCQP.QNCQPQuadLin ...
%    (A, a, a0, lambdaVec, verbosity, varargin)
%
% TODO - Allow for [] (empty) inputs
%
% Included in AToM, info@antennatoolbox.com
% (c) 2020, Lukas Jelinek, CTU in Prague, lukas.jelinek@antennatoolbox.com
% (c) 2021, Jakub Liska, CTU in Prague, jakub.liska@antennatoolbox.com
% mcode

% number of constraints
nConst = size(varargin,2)/3;

% number of unknowns
nUnknowns = size(A, 1);

% Cholesky factor of B_1 and settings of eigs routine
L = chol(varargin{1,1});
opt.cholB = 1; % matrix B is a Cholesky factor
opt.tol = 1e-10; % eigs convergence tolerance
opt.p = max(min(20, nUnknowns),ceil(0.2*nUnknowns));  % Krylov subspace


%% initialization of Lagrange's multipliers

% maximum allowed value of lambdaVec(1,1) to be in
% positive definite region (PDR)
lambdaLeadMax =  lambdaLeadMaxFun();

% check if lambdaVec is in PDR
if lambdaVec(1,1) > lambdaLeadMax
    if verbosity > 0
        disp('invalid guess')
    end
    % setup an initial Lagrange's multiplier that is in PDR
    if abs(lambdaLeadMax) < 100*eps
        lambdaVec(1,1) = - 1e-6; % adhoc treatment of zero eigenvalue
    else
        lambdaVec(1,1) = lambdaLeadMax - abs(lambdaLeadMax)/10;
    end
end

if verbosity > 1
    disp(['Proximity to PSD boundary: ', num2str( ...
        2*abs((lambdaLeadMax - lambdaVec(1,1))/...
               (lambdaLeadMax + lambdaVec(1,1))))])
end    

%% initial value of dual function

tmp = vectorSum(1);
H = A - matrixSum(1); % Hessian matrix of Lagrangian
xOpt = -0.5*mldivide(H,a - tmp); % initial solution
gOld = real(0.5*(a - tmp)'*xOpt) + ...
    (a0 - constantSum(1)); % initial value of dual function

%% plot results

% plot evolution of Lag. multipliers
if verbosity > 2
    iterCount1 = 1;
    LamPlot = [];
    LamPlot = [LamPlot, lambdaVec];
    LamFig = figure;
    for iFig = 1:size(lambdaVec, 1)
    plot(1:iterCount1, LamPlot(iFig,:), '-o')
    hold on
    end
end

% plot evolution of dual function
if verbosity > 2
    iterCount = 1;
    gPlot = [];
    gPlot = [gPlot, gOld];
    gFig = figure;
    plot(1:iterCount, gPlot, 'g-o')
end

% plot evolution of constraints
if verbosity > 2
    iterCount2 = 1;
    conPlot = [];
    
    tmpCon = nan(nConst, 1);
    for iCon = 1:nConst
        tmpCon(iCon, 1) = xOpt'*varargin{1,3*(iCon-1)+1}*xOpt + ...
            real(xOpt'*varargin{1,3*(iCon-1)+2}) + ...
            varargin{1,3*(iCon-1)+3};
    end
    
    conPlot = [conPlot, tmpCon];
    conFig = figure;
    for iFig = 1:size(lambdaVec, 1)
        plot(1:iterCount2, real(conPlot(iFig,:)), '-o')
        hold on
    end
end

%% Newton's iteration
err = Inf; % decision variable for convergence
maxNewtonIter = 100; % maximum number of Newton's steps
iter = 0; % iteration counter

while err > 1e-10 && iter < maxNewtonIter % tolerance
    iter = iter + 1; % iteration counter
    
    gradDual = zeros(nConst,1); % gradient of dual fucntion
    HessDual = zeros(nConst); % % Hessian matrix of dual function
    
    for iConst = 1:nConst
        gradDual(iConst,1) = - real((varargin{1,3*(iConst-1)+1}*xOpt + ...
           varargin{1,3*(iConst-1)+2})'*xOpt) - varargin{1,3*(iConst-1)+3};
        
        for jConst = iConst:nConst
            HessDual(iConst,jConst) = ...
                - 2*real((varargin{1,3*(iConst-1)+1}*xOpt +...
                varargin{1,3*(iConst-1)+2}/2)'* ...
                mldivide(H,(varargin{1,3*(jConst-1)+1}*xOpt + ...
                varargin{1,3*(jConst-1)+2}/2)));
        end
    end
    HessDual = HessDual + HessDual.';
    HessDual = HessDual - diag(diag(HessDual))/2;
    
    p  = - mldivide(HessDual,gradDual); % proposed Newton's shift
    
    % If the step is too large, fix it, but allow for an overshot
    if norm(p) > norm(lambdaVec)
        if verbosity > 0
        disp('Step is too large.')
        end
        p = p / norm(p)* norm(lambdaVec) / 2;
    end
    
% alpha is choosen so as to A - matrixSum(1) - alpha*matrixSumP(1) > 0, where
% matrixSum(1) is evaluated at the old lambdaVec. This forces the new
% lambdaVec to lie in the PDR.
% For alpha > 0 this gives alpha < 1/eigs(matrixSumP(1), A - matrixSum(1),...
%  1, 'lr').
% For alpha < 0 this gives alpha > 1/eigs(matrixSumP(1), A - matrixSum(1),...
% 1, 'sr').
% In the case of alpha < 0, the value alpha = 1 is acceptable - Netwon is
% propagating away from the PDR boundary.
% In the case of alpha > 0, its value is limited to alpha < 1;
    try
        opt.cholB = 1;
        alpha = 0.9*1/eigs(matrixSumP(1), chol(A - matrixSum(1)), ...
        1, 'lr', opt);
    catch ME
        switch ME.identifier
            case 'MATLAB:posdef'
                warning('Matrix is not positive definite.');
                opt.cholB = 0;
                alpha = 0.9*1/eigs(matrixSumP(1), A - matrixSum(1), ...
                1, 'lr', opt);
            otherwise
                error('Something unexpected');
        end
    end
 
    if isnan(alpha) % if eigs failed, use eig
        alpha = 0.9*1/max(real(eig(matrixSumP(1), A - matrixSum(1))));
        if verbosity > 0
            disp('Eigs failed. Full eig is used.')
        end
    end
    
    % alpha \in (0,1)
    if alpha < 0
        alpha = 1;
    else
        alpha = min(real(alpha), 1);
    end
    
    lambdaVec = lambdaVec + real(alpha*p); % proceed with the Newton's step
    
    if verbosity > 0
        disp(['Overshot control, alpha = ',num2str(alpha)])
    end
    
    if verbosity > 1
        lambdaLeadMax =  lambdaLeadMaxFun();
        disp(['Proximity to PSD boundary: ', num2str( ...
        2*abs((lambdaLeadMax - lambdaVec(1,1))/...
               (lambdaLeadMax + lambdaVec(1,1))))])
    end   
    
    % dual function after Newton's step
    tmp = vectorSum(1);
    H = A - matrixSum(1); % Hessian matrix of Lagrangian
    xOpt = -0.5*mldivide(H,a - tmp); % initil solution
    gOpt = real(0.5*(a - tmp)'*xOpt) + ...
        (a0 - constantSum(1)); % initial value of dual function
    
    % relative error
    err = 2*abs((gOld - gOpt)/(gOld + gOpt));
    if verbosity > 0
        disp(['Newton step (N = ', num2str(iter), ...
            '), relative error = ', num2str(err)])
    end
    gOld = gOpt;
    
    % plot evolution of Lag. multipliers
    if verbosity > 2
    iterCount1 = iterCount1 + 1;
    LamPlot = [LamPlot, lambdaVec];
    figure(LamFig);
    for iFig = 1:size(lambdaVec, 1)
    plot(1:iterCount1, LamPlot(iFig,:), '-o')
    hold on
    end
    hold off
    grid on
    end
    
    % plot evolution of dual function
    if verbosity > 2
        iterCount = iterCount + 1;
        gPlot = [gPlot, gOld];
        figure(gFig);
        plot(1:iterCount, gPlot, 'g-o')
        grid on
    end
    
    % plot evolution of constraints
    if verbosity > 2
        iterCount2 = iterCount2 + 1;
 
        tmpCon = nan(nConst, 1);
        for iCon = 1:nConst
            tmpCon(iCon, 1) = xOpt'*varargin{1,3*(iCon-1)+1}*xOpt + ...
                real(xOpt'*varargin{1,3*(iCon-1)+2}) + ...
                varargin{1,3*(iCon-1)+3};
        end
        
        conPlot = [conPlot, tmpCon];
        figure(conFig);
        for iFig = 1:size(lambdaVec, 1)
            plot(1:iterCount2, real(conPlot(iFig,:)), '-o')
            hold on
        end
        hold off
        grid on
    end
    
end

if iter >= maxNewtonIter
    warning('slow convergence')
end

if verbosity > 2
    figure(gFig);
    xlabel('Newton interations')
    ylabel('vaue of dual function')
    
    figure(LamFig);
    xlabel('Newton interations')
    ylabel('value of Largrange multipliers')
    
    figure(conFig);
    xlabel('Newton interations')
    ylabel('value of constraints')
end

%% results
primalOpt = real((A*xOpt + a)'*xOpt) + a0;
constVal = nan(nConst,1);

for i4 = 1:nConst
    constVal(i4,1) = real((varargin{1,3*(i4-1)+1}*xOpt + ...
        varargin{1,3*(i4-1)+2})'*xOpt) + varargin{1,3*(i4-1)+3};
end
%% Auxiliary functions

    function [OUT] = matrixSum (M)
        OUT = 0;
        for i1 = M:nConst
            OUT = OUT + lambdaVec(i1,1)*varargin{1,3*(i1-1)+1};
        end
    end

    function [OUT] = vectorSum (M)
        OUT = 0;
        for i2 = M:nConst
            OUT = OUT + lambdaVec(i2,1)*varargin{1,3*(i2-1)+2};
        end
    end

    function [OUT] = constantSum (M)
        OUT = 0;
        for i3 = M:nConst
            OUT = OUT + lambdaVec(i3,1)*varargin{1,3*(i3-1)+3};
        end
    end

    function [OUT] = matrixSumP (M)
        OUT = 0;
        for i1 = M:nConst
            OUT = OUT + p(i1,1)*varargin{1,3*(i1-1)+1};
        end
    end

    function lambdaLeadMax = lambdaLeadMaxFun % asses pos. def. region
        
        lambdaLeadMax = real(eigs(A - matrixSum(2), ...
            L, 1, 'sr', opt)); % pos. def. region
        
        % When multipliers are too far, eigs fails.
        while isnan(lambdaLeadMax)
           if verbosity > 0
               disp('Eigs failed. Full eig is used.')
           end
           lambdaLeadMax = min(real(eig(A - matrixSum(2), varargin{1,1})));
        end
    end
end