function [primarP, dualP, Ivec] = minAstB1(A, B, verbosity)
%% minAstB1 - Solve QCQP min I'AI s.t. I'BI - 1 = 0.
% This function solves quadratic program
% min {I'*A*I}
% s.t. {I'BI - 1 = 0},
% where matrices are hermitian and B is positive definite.
%
%  INPUTS
%    A: hermitian matrix [n x n]
%    B: positive definite hermitian matrix [n x n]
%    verbosity: reporting flag (true -> report, false -> do not report)
%
%  OUTPUTS
%    primarP: value of primar problem
%    dualP: value of dual problem
%    Ivec: argmin
%
%  SYNTAX
%
%   [primarP, dualP, Ivec] = minAstB1 (A, B, verbosity)
%
% AToM package Fundamental Bounds. /info@antennatoolbox.com/
% (c) 2020, Jakub Liska, CTU in Prague, /jakub.liska@antennatoolbox.com/
% mcode

% eigs settings
nUnknowns = size(A,1);
if nUnknowns > 10
    % Krylov subspace dimension modification
    opts.p = max(min(20, nUnknowns), ceil(0.3*nUnknowns));
end
opts.spdB = 1; % positive definite matrix
opts.disp = verbosity; % reporting flag of eigs function

A = 0.5*(A + A');
B = 0.5*(B + B');

normA = norm(A);
normB = norm(B);

A = A/normA;
B = B/normB;
coef = normA/normB;

[Ivec, dualP, flag] = eigs(A, B, 1, 'sr', opts); % problem solution
if flag % eigs did not converge
    if verbosity == 1
        warning('Use of full eig, eigs did not converge.');
    end
    [Imat, D] = eig(A, B, 'chol', 'vector');
    [dualP, ind] = min(D);
    Ivec = Imat(:, ind);
end
dualP = coef*dualP;
primarP = coef*real(Ivec'*A*Ivec)/real(Ivec'*B*Ivec); % primar problem
end