function [primarP, dualP, Ivec, lambda, dualGap] = minAstBn(A, lambda, verbosity, varargin)
%% minAstBm - Solve QCQP min I'AI s.t. I'B1I - 1 = 0, I'BiI + betai = 0.
% This function solves quadratic program
% min {I'*A*I}
% s.t. {I'B1I - 1 = 0}
%      {I'BiI + betai = 0}, i = 2...nCnstr
% All matrices are hermitian and matrix B1 is hermitian positive definite
% matrix. Minimal number of constraints is 2.
%
%  INPUTS
%   A: hermitian matrix [n x n]
%   lambda: initial Lagrange's multipliers [1 x nConstr]
%   verbosity: reporting flag (0 -> do not report, 1 -> report,
%     2 -> detailed report)
%   varargin: triplets of
%     matrix Bi,
%     vector bi replaced by empty [],
%     scalar betai
%
%  OUTPUTS
%   primarP: value of primar problem
%   dualP: value of dual problem
%   Ivec: argmin
%   lambda: optimal Lagrange's multipliers
%   dualGap: dual gap flag, relative toleranc of dual and primar problem
%     difference is larger then 1e-4
%
%  SYNTAX
%
%   [primarP, dualP, Ivec, lambda, dualGap] = solvers.QCQP.minAstBn(...
%     A, lambda, verbosity, varargin);
%
% AToM package Fundamental Bounds, info@antennatoolbox.com
% (c) 2021, Jakub Liska, CTU in Prague, jakub.liska@antennatoolbox.com
% mcode

% unpack cell in varargin
if all(size(varargin) == 1) && iscell(varargin{1})
    varargin = varargin{1};
end

nDim = size(A); % matrices dimensions
nUnknowns = nDim(1); % Ivec length
nConstr = size(varargin,2)/3; % number of constraints
nPar = nConstr - 1; % number of independent parameters

% Cholesky decomposition of positive definite matrix
cholFacB = chol(varargin{1});

opts.tol = 1e-11; % eigs convergence tolerance
opts.p = max(min(20, nUnknowns), ceil(0.3*nUnknowns)); % Krylov subspace

mat_ind = @(Mind) 3*(Mind - 1) + 1; % matrices indexes
const_ind = @(rInd) 3*rInd; % constants indexes
con_ind = @(parInd) parInd + 1; % parameter index -> constraint index

% matrices transformation
mat_symmetrization = @(M) 0.5*(M+M');
mat_transform = @(M) mat_symmetrization((cholFacB')\M/cholFacB);
A = mat_transform(A);
varargin{1} = eye(nDim);
for indT = 2:nConstr
    varargin{mat_ind(indT)} = mat_transform(varargin{mat_ind(indT)});
end

%% fminsearch
if nConstr <= 4
    tolF = 1e-9;
    tolX = 1e-6;
    maxIter = 2e3;
else
    tolF = 1e-6;
    tolX = 1e-5;
    maxIter = 1e3;
end

options = optimset('TolFun', tolF, 'MaxIter', maxIter, 'MaxFunEvals', 1e5,...
    'TolX', tolX);
if verbosity > 0
    disp(...
        ['fminsearch computation with function tolerance ', num2str(tolF),...
        ', multiplicator tolerance ', num2str(tolX)...
        ' and maximal number of iterations ', num2str(maxIter)]);
    if verbosity == 2
        options = optimset('Display', 'iter', 'PlotFcns', @optimplotfval,...
            'TolFun', tolF , 'MaxIter', maxIter, 'MaxFunEvals', 1e5, 'TolX', tolX);
    end
    tic;
end
[nuF, dualP] = fminsearch(@(mult) -dual_problem(mult), lambda(2:end), options);
if verbosity > 0
    fMinSearchTime = toc;
    disp(['fminsearch computation time: ', num2str(fMinSearchTime), ' s']);
end

%% Results
if verbosity > 0
    disp('Final evaluation');
end

dualP = -dualP; % dual problem value

% multiple algebraic eigenvalues
sumMat = A - sum_mat(nuF);
tolEig = 1e-3;
[~, dMin, nanFlag] = eigs(sumMat, 1, 'sr', opts);

if nanFlag  % eigs did not converge
    [dMin, IvecList] = full_eig(sumMat, tolEig);
else
    r = rank(sumMat - dMin*eye(nUnknowns), abs(dMin)*tolEig);
    nEig = nUnknowns - r;
    if nEig <= (opts.p + 1)
        opts.p = ceil(0.7*(nEig + 1) + 0.3*nUnknowns);
    end
    [IvecList, ~, nanFlag] = eigs(sumMat, nEig, 'sr', opts);
    if nanFlag  % eigs did not converge
        [dMin, IvecList] = full_eig(sumMat, tolEig);
    end
end

lambda = [dMin, nuF]; % values of multipliers

%% Symmetry treatement
% same eigenvalues
eigAlgebMultip = size(IvecList,2);
if eigAlgebMultip >= 2
    [primarP, Ivec] = symmetry_correction(); % mixing eigenvectors
else
    Ivec = sqrt(- varargin{3})*IvecList;
    primarP = real(Ivec'*A*Ivec); % primar function value
end

Ivec = cholFacB\Ivec; % vector transformation

%% dual gap detection
dg = 2*abs(primarP - dualP)/abs(primarP + dualP);
if dg > 1e-4
    if nargout == 5
        dualGap = true;
    else
        warning(['Dual gap: ', num2str(dg)]);
    end
else
    dualGap = false;
end

% END of function minAstBn

%% FUNCTIONS -------------------------------------------------------------------
    function M = sum_mat(mult)
        %% constraints matrices summation
        %  INPUTS:
        %     mult: independent multipliers
        %  OUTPUTS
        %     M: sum of matrices of constraints multiplied by independent
        %        multipliers
        
        M = zeros(nDim) + 1i*zeros(nDim); % initialization
        
        for ind = 2:nConstr
            M = M + mult(ind-1)*varargin{mat_ind(ind)};
        end
    end

    function dP = dual_problem(mult)
        %% dual function
        %  INPUTS
        %     mult: independent multipliers
        %  OUTPUTS
        %     dP: dual function value
        
        sumM = A - sum_mat(mult);
        [~, lam, nF] = eigs(sumM, 1, 'sr', opts);
        if nF % eigs did not converge
            Lam = eig(sumM);
            lam = min(Lam);
            if verbosity > 1
                warning('Use of full eig, eigs did not converge.');
            end
        end
        dP = -varargin{3}*lam; % dual function value
        for indPar = 1:nPar
            dP = dP - mult(indPar)*varargin{const_ind(con_ind(indPar))};
        end
        dMin = lam; % save to global variable
    end

    function [pP, I] = symmetry_correction()
        %% symmetry correction
        %  OUTPUTS
        %     pP: primar problem value
        %     I: mixed final eigenvector
        
        % initialization
        mappedMatrices = nan(eigAlgebMultip, eigAlgebMultip, nConstr);
        constraintVec = nan(nConstr, 1);
        
        for iCon = 1:nConstr
            % matrix mapping to eigenvector space
            mappedMatrices(:,:, iCon) = IvecList'*varargin{mat_ind(iCon)}*IvecList;
            % constraint vector composition
            constraintVec(iCon) = varargin{const_ind(iCon)};
        end
        
        % mixing vector initialization
        aVecF = zeros(eigAlgebMultip,1);
        aVecF(1) = 1;
        
        % constraint fitting
        conOpt = optimset('TolFun', 1e-14, 'TolX', 1e-14, 'MaxIter', 1e4,...
            'MaxFunEvals', 1e5);
        if verbosity > 0
            disp(...
                'symmetry treatment: constraint fitting with tolerance 1e-15 and maximal number of iterations 1e4'...
                );
            if verbosity == 2
                conOpt = optimset('Display', 'iter', 'PlotFcns', @optimplotfval,...
                    'TolFun', 1e-14, 'TolX', 1e-14, 'MaxIter', 1e4,...
                    'MaxFunEvals', 1e5);
            end
        end
        aVecF = fminsearch(@(aVec) constraint_fitting(...
            mappedMatrices, constraintVec, aVec), aVecF, conOpt);
        
        % mixing final eigenvector
        I = IvecList*aVecF;
        
        % primar problem evaluation
        pP = real(I'*A*I);
    end

    function val = constraint_fitting(mMat, conVec, aVec)
        %% constraint fitting function
        %  INPUTS
        %     mMat: mapped constraint matrices
        %        [eigAlgMultip x eigAlgMultip x nconstr]
        %     conVec: vector of constraints constants
        %     aVec: mixing vector
        %  OUTPUTS
        %     val: matric of constraint fitting
        
        for iConstr = 1:nConstr % constraint evaluation
            conVec(iConstr) = conVec(iConstr) + aVec'*mMat(:,:, iConstr)*aVec;
        end
        val = norm(conVec); % fitting norm
    end

    function [dMin, Ilist] = full_eig(Amat, tol)
        %% eig function for same smallest eigenvalues and vectors
        %  INPUTS:
        %     Amat: matrix
        %     tol: eigenvalue diference tolerance
        %  OUTPUTS:
        %     dMin: minimal eigenvalue
        %     Ilist: eigenvectors
        
        cmpr_fun = @(minV, cmprV) abs(minV - cmprV)./abs(minV) < tol;
        if verbosity > 1
            warning('Use of full eig, eigs did not converge.');
        end
        [Imat, D] = eig(Amat, 'vector');
        dMin = min(D);
        sameEigV = cmpr_fun(dMin, D);
        Ilist = reshape(Imat(... choose corespondin eigenvectors
            repmat(sameEigV.', nUnknowns, 1)),... % logical indexing
            nUnknowns, sum(sameEigV)); % reshape to vectors
    end
end