function Hess = nHess(f, x, type)
%% nHess - numerical Hess matrix
% The function evaluates numerically function Hess matrix by central
% difference.
%
%   INPUTS:
%    f: function
%    x: point
%
%   OUTPUTS:
%    Hess: Hess matrix
%
%   SYNTAX
%
%    Hess = solvers.derivatives.nHess(f, x)
%
% AToM package Fundamental Bounds, info@antennatoolbox.com
% © 2020, Jakub Liska, CTU in Prague, jakub.liska@antennatoolbox.com
% mcode

n = length(x);
Hess = nan(n);

for ind = 1:n
    h = eps(x(ind))^(1/3); % optima shifts
    
    x1 = x;
    x2 = x;
    
    x1(ind) = x(ind) - h;
    x2(ind) = x(ind) + h;
    
    gradx1 = solvers.derivatives.nGrad(f, x1, type)';
    gradx2 = solvers.derivatives.nGrad(f, x2, type)';
    
    if nargin > 2 && strcmp(type, 'complex')
        y1 = x;
        y2 = x;
        
        y1(ind) = x(ind) - 1i*h;
        y2(ind) = x(ind) + 1i*h;
        
        grady1 = solvers.derivatives.nGrad(f, y1, type)';
        grady2 = solvers.derivatives.nGrad(f, y2, type)';
        
    else
        grady1 = zeros(1,n);
        grady2 = zeros(1,n);
    end
    
    Hess(ind, :) = 0.5*(gradx2 - gradx1)/(2*h) +...
        0.5*1i*(grady2 - grady1)/(2*h);
end
Hess = 0.5*(Hess + Hess');
end