function [x1, x2] = calcPSDregion(PSDmat, NDmat)
%% calcPSDregion - calculates PSD region of PSDmat + x*NDmat
% This function determines positive definite region of matrix PSDmat + x*NDmat
% with respect to x.
%
%  INPUTS:
%   PSDmat: positive definite matrix [n x n]
%   NDmat: Hermitian matrix [n x n]
%
%  OUTPUTS:
%   x1: PSD region left-bound
%   x2: PSD region right-bound
%
%  SYNTAX
%
%   [x1, x2] = calcPSDregion(PSDmat, NDmat)
%
% AToM package Fundamental Bounds, info@antennatoolbox.com
% (c) 2020, Jakub Liska, CTU in Prague, jakub.liska@antennatoolbox.com
% mcode

% eigs settings
opts.spdB = 1; % positive definite matrix
opts.p = ceil(0.3*size(PSDmat,1)); % Krylov subspace

[~, D, flag] = eigs(NDmat, PSDmat, 2, 'bothendsreal', opts);
if flag % eigs did not converge
   warning('Use of full eig, eigs did not converge.');
   D = eig(NDmat, PSDmat, 'chol');
   d = [max(D), min(D)];
else
   d = diag(D);
end

x = - 1./d([2,1]);

if x(2) < 0 % NDmat is positive definite
   x1 = x(1);
   x2 = inf;
elseif x(1) > 0 % NDmat is negative definite
   x1 = - inf;
   x2 = x(2);
else
   x1 = x(1);
   x2 = x(2);
end
end