function vectors = payOffTable(verbosity, varargin)
%% payOffTable - Finde the best solutions according to given metrics.
% This function finds the pareto-optimal points, which are always the best in
% one metric with given constraints. Zero matrices and vectors inesert as empty
% cell {}.
%  INPUTS
%   verbosity: reporting flag (0 -> do not report, 1 -> report,
%     2 -> detailed report)
%   varargin: quintets of matrix, vector, scalar and
%    flag (0: constraint,
%     +-1: optimized metric; negative -> minimize, positive -> maximize),
%    connection vector (same numbers connects optimized metric to positive
%     definite constraint and vice versa, empty [] is connected to all)
%
%  OUTPUTS
%   vectors: corresponding to pareto-optimal points
%
%  SYNTAX
%
%  vectors = solvers.payOffTable(verbosity,...
%     M1, V1, s1, f1, c1,...
%     M2, V2, s2, f2, c2,...
%     M3, V3, s3, f3, c3,...
%
% AToM package Fundamental Bounds, info@antennatoolbox.com
% (c) 2020, Jakub Liska, CTU in Prague, jakub.liska@antennatoolbox.com
% mcode

setN = 5; % quintets
nVarargin = nargin-1;

if mod(nVarargin, setN) % input control
    error('Wrong number of inputs in pay off table function.');
end

nFun = nVarargin/setN; % number of functions

% indexing functions
line_start = @(index) (index - 1).*setN;
mat_ind = @(index) line_start(index) + 1; % matrix index
vec_ind = @(index) line_start(index) + 2; % vector index
scalar_ind = @(index) line_start(index) + 3; % scalar index
type_ind = @(index) line_start(index) + 4; % type index
connect_ind = @(index) line_start(index) + 5; % connector index
appConstr_ind = @(index) reshape(... triplet indices
    [mat_ind(index); vec_ind(index); scalar_ind(index)], 1, []);

for iRow = 1:nFun
    if ~isempty(varargin{mat_ind(iRow)})
        nUnknowns = size(varargin{mat_ind(iRow)},2);
        break;
    end
end

% objective functions
optMetLog = logical(cell2mat(varargin(type_ind(1:nFun))));
funVec = 1:nFun;
metVec = funVec(optMetLog); % objective functions indices
constrVec = funVec(~optMetLog); % constraint indices
nMet = sum(optMetLog); % number of optimized functions
nConstr = nFun - nMet; % number of constraints

vectors = nan(nUnknowns,nMet); % output initialization

% linear constraints
if isempty(cell2mat(varargin(vec_ind(constrVec))))
    linTF = false;
else
    linTF = true;
end

% QCQP
met = 0;
for iMet = metVec % objective function index
    met = met + 1;
    appConstr = nan(1,nConstr); % initialization
    nConstr = 0;
    for iConstr = constrVec % constraint index
        posTF = any(cell2mat(varargin(connect_ind(iConstr))) ==...
            varargin{connect_ind(iMet)}); % normalizing constraint
        % assign constraint
        if  posTF || isempty(varargin{connect_ind(iConstr)})
            nConstr = nConstr + 1;
            % normalizing constraint first
            if posTF && nConstr > 1
                appConstr(2:nConstr) = appConstr(1:nConstr-1);
                appConstr(1) = iConstr;
            else
                appConstr(nConstr) = iConstr;
            end
        end
    end
    appConstr(isnan(appConstr)) = [];
    
    if ~isempty(varargin{vec_ind(iMet)}) || linTF % with linear terms
        [~, ~, vectors(:,met)] = solvers.QCQP.QNCQPQuadLin(...
            - sign(varargin{type_ind(iMet)})*varargin{mat_ind(iMet)},...
            - sign(varargin{type_ind(iMet)})*varargin{vec_ind(iMet)},...
            - sign(varargin{type_ind(iMet)})*varargin{scalar_ind(iMet)},...
            zeros(1,nConstr), verbosity,...
            varargin(appConstr_ind(appConstr)));
    elseif nConstr == 1 % one quadratic constraint
        [~, ~, vectors(:,met)] = solvers.QCQP.minAstB1(...
            - sign(varargin{type_ind(iMet)})*varargin{mat_ind(iMet)},...
            varargin{mat_ind(appConstr)}./varargin{scalar_ind(appConstr)},...
            verbosity);
    else % more quadratic constraints
        [~, ~, vectors(:,met)] = solvers.QCQP.minAstBn(...
            - sign(varargin{type_ind(iMet)})*varargin{mat_ind(iMet)},...
            zeros(1,nConstr), verbosity,...
            varargin(appConstr_ind(appConstr)));
    end
end
end