% The script computes trapping Parteto-optimal point for a spherical shell,
% see DOI: XXXXX for more details. The result coresponds to black cross in
% Fig. 2 and current density in Fig. 3.

% Included in AToM, info@antennatoolbox.com
% (c) 2022, Jakub Liska, CTU FEE, jakub.liska@antennatoolbox.com

clc, clear, close all;

%% mesh - spherical shell
% Lebedev sampling
% N could be 6, 14, 26, 38, 50, 74, 86, 110, 146, 170, 194, 230, 266, 
% 302, 350, 434, 590, 770, 974, 1202, 1454, 1730, 2030, 2354, 2702, ...
N = 230;
lebPoints = models.utilities.thirdParty.getLebedevSphere(N);
DT = delaunayTriangulation(lebPoints(:,1),lebPoints(:,2),lebPoints(:,3));

% get nodes, connectivityList
nodes = DT.Points;
connectivityList = DT.convexHull;

% create mesh structure
mesh = models.utilities.meshPublic.getMeshData2D(nodes, connectivityList);

% plot mesh
options.showEdges = true;
options.showTriangles = true;
hndlMesh = results.plotMesh(mesh, 'options', options);
view(25,45);
drawnow;

%% setup
r = [0; 0; 0]; % trapping point
rho = mesh.nodes - r'; % distance to all mesh points
rho = min(vecnorm(rho')); % radius of largest inscribed sphere

%% EFIE operators
ka = 1e-2; % electrical size
Zs = 0.01; % surface resistivity
quadOrder = 7; % MoM quadrature order
a = mesh.normDistanceA; % radius of the smallest circumscribing sphere
mu0 = models.utilities.constants.mu0(); % permeability of vacuum
f = ka*models.utilities.constants.c0()/(2*pi*a); % frequency
m.type = 'Bfield'; % dipole properties

[OP, ~] = models.utilities.matrixOperators.MoM2D.batch.evaluate(mesh, ...
    f, Zs, 'normalize', false, 'symmetrize', true,...
    'quadOrder', quadOrder, 'requests', {'Rmat', 'Xe', 'Xm'}, ...
    'usegpu', true); % compute operators

norm_mat = @(M) M/norm(M); % normalization function

%% solenoidal currents
[IloopStar, d] = eig(OP.Xe, OP.Xm, 'vector'); % GEP

figure(); % plot eigenvalues
semilogy(abs(d)', 'o'); title('Loop-Star Decomposition');
grid on
drawnow;

iSol = IloopStar(:,abs(d) < 1); % basis separation (loops are selected)
Rmat = iSol'*OP.Rmat*iSol; % lost power operator in the loop basis
Rmat = 0.5*(Rmat + Rmat'); % symmetrization

%% B-field operators
[Bsol, BsolDer, BsolDer2] = getBfieldOperators(mesh,...
    OP, r, quadOrder, iSol);

BopMat = [Bsol(3,:);... Omega matrix
    rho*BsolDer.x(1:2,:);...
    rho*BsolDer.y(1:2,:);...
    rho*BsolDer.z(1:2,:);...
    rho^2*BsolDer2.xx(3,:);...
    rho^2*BsolDer2.xy(3,:);...
    rho^2*BsolDer2.xz(3,:);...
    rho^2*BsolDer2.yy(3,:);...
    rho^2*BsolDer2.yz(3,:);...
    rho^2*BsolDer2.zz(3,:)];

[V, d] = eig(BopMat'*BopMat, Rmat, 'chol', 'vector'); % GEP
nVec = sum(d/max(d) > 1e-14); % basis dimension
iBop = V(:,end-nVec+1:end); % basis separation

figure(); % plot eigenvalues
semilogy(abs(d), 'xr'); title('Omega problem');
grid on
drawnow;

[Brdc, BrdcDer, BrdcDer2, Rrdc] = transformOperators(iBop,...
    Bsol, BsolDer, BsolDer2, Rmat); % transform all operators

%% F zeros
Lred = chol(Rrdc); % lost power Cholesky decomposition
FxyNull = null([Brdc(1:2,:);... % null space
    BrdcDer.x(3,:);...
    BrdcDer.y(3,:);...
    BrdcDer.z(3,:)]/Lred);
iFxyNull = Lred\FxyNull; % lost power orthogonal nullspace

[Bfxy, BfxyDer, BfxyDer2, Rfxy] = transformOperators(iFxyNull,...
    Brdc, BrdcDer, BrdcDer2, Rrdc); %transfor all operatos to new basis

% get operators Phi, Gamma, Xi
[Pf, Gf, Xf] = getUoperators(Bfxy, BfxyDer, BfxyDer2);

%% max xi
[~, normTriplets] = solvers.normalizeTriplet(... normalize operators
    Rfxy, [], -0.5,... 1
    -Xf.xx, [], 0,... 4
    Xf.yy - Xf.xx, [], 0,... 7
    Xf.xx - Xf.zz, [], 0,... 10
    Xf.xy, [], 0,... 13
    Xf.xz, [], 0,... 16
    Xf.yz, [], 0); % 19

verb = 1; % optimization verbosity
nu = zeros(1,3); % initial lagrange multipliers
[primarP, dualP, Iopt] = solvers.QCQP.minAstBn(... maximize xi
    normTriplets{4}, nu, verb,...
    normTriplets{[1:3, 7:12]});

[Phi, Gamma, Xi] = evalU(Iopt, m, Pf, Gf, Xf); % get Phi, Gamma, Xi
Plost = 0.5*Iopt'*Rfxy*Iopt; % lost power
PhiN = Zs*Phi/Plost*rho^2/mu0^2; % normalized Phi
xiN = Zs*eigs(Xi, 1, 'sr')/Plost*rho^4/mu0^2; % normalized Xi

%% results
data = [xiN./(2*sqrt(PhiN)), sqrt(PhiN)]; % quantities in Fig. 2
figure(); % plotting
loglog(data(:,1), data(:,2), 'x');
xlabel('$\xi_n/(2 \sqrt{\Phi_n)}$', 'Interpreter','latex');
ylabel('$\sqrt{\Phi_n}$', 'Interpreter','latex');
grid on;
title('Pareto-optimal point, Fig. 2');

% revers transforamtion of currents
IbOP = iFxyNull*Iopt; % force zeroing problem
Irwg = iSol*iBop*IbOP; % Omega problem and loop-star decomposition

hndlOpt = results.plotCurrent(mesh,... plot current
    'basisFcns', OP.BF, 'Ivec', Irwg,...
    'part', 'abs', 'scale', 'linear', 'arrowScale', 'proportional');

%% functions

function [Bmat, BmatDer, BmatDer2] = getBfieldOperators(...
    Mesh, OP, r, nQuad, iBasis)
%% getBfieldOperators - computes B-field operators
% The function computes B-field operator, B-field first and second
% derivative operators.
%
%   INPUTS:
%    Mesh: struct
%    OP: struct
%    r: coordinates in space [3 x 1]
%    nQuad: number of quadrature points (1-12)
%    iBasis: B-field operator basis
%
%   OUTPUTS:
%    Bmat: B-field operator [3 x m]
%    BmatDer: B-field first derivative operators structure (x, y, z)
%    BmatDer2: B-field second derivative operators structure (xx, xy,
%       xz, yy, yz, zz)
%
%   SYNTAX
%    [BfieldMat, BfieldMatDer, BfieldMatDer2, iBasisB] = getBfieldOperators(...
%       Mesh, OP, r, nQuad, iBasis)
%
% Jakub Liska, FEE CTU in Prague, jakub.liska@fel.cvut.cz

% quad points
[pQuadX, pQuadY, pQuadZ, w, nQuadPoints, ~] = ...
    models.utilities.simplexIntegration.triaQuad(...
    Mesh.nodes(Mesh.connectivityList(:,1),:),...
    Mesh.nodes(Mesh.connectivityList(:,2),:),...
    Mesh.nodes(Mesh.connectivityList(:,3),:),...
    nQuad);

%% initialization
% field
Bmat = zeros(3,OP.BF.nUnknowns);

% first derivative
BmatDer.x = Bmat; % dB/dx
BmatDer.y = Bmat; % dB/dy
BmatDer.z = Bmat; % dB/dz

% second derivative
BmatDer2.xx = Bmat; % d^2B/dx^2
BmatDer2.xy = Bmat; % d^2B/dxdy
BmatDer2.xz = Bmat; % d^2B/dxdz

BmatDer2.yy = Bmat; % d^2B/dy^2
BmatDer2.yz = Bmat; % d^2B/dydz

BmatDer2.zz = Bmat; % d^2B/dz^2

% basis functions
rho_plus = @(V,r) (r-V);
rho_minus = @(V,r) - rho_plus(V,r);
basis_fcn = @(L,rho) L*rho;

%% solution for each basis function
for indBF=1:OP.BF.nUnknowns
    % data
    triaP = OP.BF.data(indBF, OP.BF.Col.trianglePlus);
    triaM = OP.BF.data(indBF, OP.BF.Col.triangleMinus);

    nodeP = Mesh.nodes(OP.BF.data(indBF, OP.BF.Col.nodePlus),:).';
    nodeM = Mesh.nodes(OP.BF.data(indBF, OP.BF.Col.nodeMinus),:).';

    eLnght = Mesh.triangleEdgeLengths(OP.BF.data(indBF, OP.BF.Col.edge));

    % quadrature points
    for indQp = 1:nQuadPoints
        pP = [pQuadX(triaP, indQp); pQuadY(triaP, indQp); pQuadZ(triaP, indQp)];
        pM = [pQuadX(triaM, indQp); pQuadY(triaM, indQp); pQuadZ(triaM, indQp)];

        Rp = r - pP;
        normRp = norm(Rp);
        normRp3rec = normRp^(-3);
        normRp5rec = normRp^(-5);
        normRp7rec = normRp^(-7);

        Rm = r - pM;
        normRm = norm(Rm);
        normRm3rec = normRm^(-3);
        normRm5rec = normRm^(-5);
        normRm7rec = normRm^(-7);

        bfP = basis_fcn(eLnght, rho_plus(nodeP, pP));
        bfM = basis_fcn(eLnght, rho_minus(nodeM, pM));

        %% B-field
        Bmat(:,indBF) = Bmat(:,indBF) +...
            w(indQp)*cross(bfP, Rp)*normRp3rec +...
            w(indQp)*cross(bfM, Rm)*normRm3rec;

        if nargout > 1
            %% B-field derivative
            % [xx, yy, zz]
            derDiagP = normRp3rec - 3*normRp5rec*Rp.^2;
            derDiagM = normRm3rec - 3*normRm5rec*Rm.^2;

            % xy
            der12P = - 3*normRp5rec*Rp(1)*Rp(2);
            der12M = - 3*normRm5rec*Rm(1)*Rm(2);

            % xz
            der13P = - 3*normRp5rec*Rp(1)*Rp(3);
            der13M = - 3*normRm5rec*Rm(1)*Rm(3);

            % yz
            der23P = - 3*normRp5rec*Rp(2)*Rp(3);
            der23M = - 3*normRm5rec*Rm(2)*Rm(3);

            BmatDer.x(:,indBF) = BmatDer.x(:,indBF) +...
                w(indQp)*cross(bfP, [derDiagP(1); der12P; der13P]) +...
                w(indQp)*cross(bfM, [derDiagM(1); der12M; der13M]);

            BmatDer.y(:,indBF) = BmatDer.y(:,indBF) +...
                w(indQp)*cross(bfP, [der12P; derDiagP(2); der23P]) +...
                w(indQp)*cross(bfM, [der12M; derDiagM(2); der23M]);

            BmatDer.z(:,indBF) = BmatDer.z(:,indBF) +...
                w(indQp)*cross(bfP, [der13P; der23P; derDiagP(3)]) +...
                w(indQp)*cross(bfM, [der13M; der23M; derDiagM(3)]);

            if nargout > 2
                %% B-field second derivative
                % x dx dx, y dy dy, z dz dz
                dderDiagP = 15*normRp7rec*Rp.^3 - 9*normRp5rec*Rp;
                dderDiagM = 15*normRm7rec*Rm.^3 - 9*normRm5rec*Rm;

                % x dy dz, y dx dz, z dx dy
                dderOffP = 15*normRp7rec*prod(Rp);
                dderOffM = 15*normRm7rec*prod(Rm);

                % x dx dy, y dx dx
                xdxdyP = 15*normRp7rec*Rp(1)^2*Rp(2) - 3*normRp5rec*Rp(2);
                xdxdyM = 15*normRm7rec*Rm(1)^2*Rm(2) - 3*normRm5rec*Rm(2);

                % x dx dz, z dx dx
                xdxdzP = 15*normRp7rec*Rp(1)^2*Rp(3) - 3*normRp5rec*Rp(3);
                xdxdzM = 15*normRm7rec*Rm(1)^2*Rm(3) - 3*normRm5rec*Rm(3);

                % x dy dy, y dx dy
                xdydyP = 15*normRp7rec*Rp(2)^2*Rp(1) - 3*normRp5rec*Rp(1);
                xdydyM = 15*normRm7rec*Rm(2)^2*Rm(1) - 3*normRm5rec*Rm(1);

                % x dz dz, z dx dz
                xdzdzP = 15*normRp7rec*Rp(3)^2*Rp(1) - 3*normRp5rec*Rp(1);
                xdzdzM = 15*normRm7rec*Rm(3)^2*Rm(1) - 3*normRm5rec*Rm(1);

                % y dy dz, z dy dy
                ydydzP = 15*normRp7rec*Rp(2)^2*Rp(3) - 3*normRp5rec*Rp(3);
                ydydzM = 15*normRm7rec*Rm(2)^2*Rm(3) - 3*normRm5rec*Rm(3);

                % y dz dz, z dy dz
                ydzdzP = 15*normRp7rec*Rp(3)^2*Rp(2) - 3*normRp5rec*Rp(2);
                ydzdzM = 15*normRm7rec*Rm(3)^2*Rm(2) - 3*normRm5rec*Rm(2);

                BmatDer2.xx(:,indBF) = BmatDer2.xx(:,indBF) +...
                    w(indQp)*cross(bfP, [dderDiagP(1); xdxdyP; xdxdzP]) +...
                    w(indQp)*cross(bfM, [dderDiagM(1); xdxdyM; xdxdzM]);
                BmatDer2.xy(:,indBF) = BmatDer2.xy(:,indBF) +...
                    w(indQp)*cross(bfP, [xdxdyP; xdydyP; dderOffP]) +...
                    w(indQp)*cross(bfM, [xdxdyM; xdydyM; dderOffM]);
                BmatDer2.xz(:,indBF) = BmatDer2.xz(:,indBF) +...
                    w(indQp)*cross(bfP, [xdxdzP; dderOffP; xdzdzP]) +...
                    w(indQp)*cross(bfM, [xdxdzM; dderOffM; xdzdzM]);

                BmatDer2.yy(:,indBF) = BmatDer2.yy(:,indBF) +...
                    w(indQp)*cross(bfP, [xdydyP; dderDiagP(2); ydydzP]) +...
                    w(indQp)*cross(bfM, [xdydyM; dderDiagM(2); ydydzM]);
                BmatDer2.yz(:,indBF) = BmatDer2.yz(:,indBF) +...
                    w(indQp)*cross(bfP, [dderOffP; ydydzP; ydzdzP]) +...
                    w(indQp)*cross(bfM, [dderOffM; ydydzM; ydzdzM]);

                BmatDer2.zz(:,indBF) = BmatDer2.zz(:,indBF) +...
                    w(indQp)*cross(bfP, [xdzdzP; ydzdzP; dderDiagP(3)]) +...
                    w(indQp)*cross(bfM, [xdzdzM; ydzdzM; dderDiagM(3)]);
            end
        end
    end
end
cnst = models.utilities.constants.mu0()/4/pi;

if nargin > 4
    f_ = @(M) cnst*(M*iBasis);
else
    f_= @(M) cnst*M;
end

Bmat = f_(Bmat);

if nargout > 1
    BmatDer.x = f_(BmatDer.x);
    BmatDer.y = f_(BmatDer.y);
    BmatDer.z = f_(BmatDer.z);

    if nargout > 2
        BmatDer2.xx = f_(BmatDer2.xx);
        BmatDer2.xy = f_(BmatDer2.xy);
        BmatDer2.xz = f_(BmatDer2.xz);
        BmatDer2.yy = f_(BmatDer2.yy);
        BmatDer2.yz = f_(BmatDer2.yz);
        BmatDer2.zz = f_(BmatDer2.zz);
    end
end
end

function [Btran, BderTran, Bder2Tran, Rtran] = transformOperators(iBasis,...
    Bmat, BmatDer, BmatDer2, Rmat)
%% Function transforms all operators.

transform_ = @(M) M*iBasis; % transformation function

Btran = transform_(Bmat); % B

BderTran.x = transform_(BmatDer.x); % Bder
BderTran.y = transform_(BmatDer.y);
BderTran.z = transform_(BmatDer.z);

Bder2Tran.xx = transform_(BmatDer2.xx); % Bder2
Bder2Tran.xy = transform_(BmatDer2.xy);
Bder2Tran.xz = transform_(BmatDer2.xz);
Bder2Tran.yy = transform_(BmatDer2.yy);
Bder2Tran.yz = transform_(BmatDer2.yz);
Bder2Tran.zz = transform_(BmatDer2.zz);

Rtran = iBasis'*transform_(Rmat); % lost power
Rtran = 0.5*(Rtran + Rtran'); % symmetrization
end

function [Umat, Fmat, Hmat] = getUoperators(BfieldMat,...
    BfieldMatDer, BfieldMatDer2)
%% getBfieldOperators - computes potential energy operators
% The function computes potential energy operator, force operators and Hess
% matrix operators.
%
%   INPUTS:
%    BfieldMat: B-field operator [3 x m]
%    BfieldMatDer: B-field first derivative operators structure (x, y, z)
%    BfieldMatDer2: B-field second derivative operators structure (xx, xy,
%       xz, yy, yz, zz)
%
%   OUTPUTS:
%    Umat: potential energy operator [m x m]
%    Fmat: force operators structur(x, y, z)
%    Hmat: Hess matrix elements operators (xx, xy, xz, yy, yz, zz)
%
%   SYNTAX
%
%    Umat = getUoperators(BfieldMat)
%    [Umat, Fmat] = getUoperators(BfieldMat, BfieldMatDer)
%    [Umat, Fmat, Hmat] = getUoperators(BfieldMat,...
%       BfieldMatDer, BfieldMatDer2)
%
% Jakub Liska, FEE CTU in Prague, jakub.liska@fel.cvut.cz

% Energy
Umat = BfieldMat'*BfieldMat;

if nargout > 1
    % Force
    Fmat.x = -BfieldMat'*BfieldMatDer.x;
    Fmat.y = -BfieldMat'*BfieldMatDer.y;
    Fmat.z = -BfieldMat'*BfieldMatDer.z;

    Fmat.x = Fmat.x + Fmat.x';
    Fmat.y = Fmat.y + Fmat.y';
    Fmat.z = Fmat.z + Fmat.z';

    if nargout > 2
        % Hess matrix
        Hmat.xx = BfieldMatDer2.xx'*BfieldMat +...
            BfieldMatDer.x'*BfieldMatDer.x;
        Hmat.xy = BfieldMatDer2.xy'*BfieldMat +...
            BfieldMatDer.y'*BfieldMatDer.x;
        Hmat.xz = BfieldMatDer2.xz'*BfieldMat +...
            BfieldMatDer.z'*BfieldMatDer.x;

        Hmat.yy = BfieldMatDer2.yy'*BfieldMat +...
            BfieldMatDer.y'*BfieldMatDer.y;
        Hmat.yz = BfieldMatDer2.yz'*BfieldMat +...
            BfieldMatDer.z'*BfieldMatDer.y;

        Hmat.zz = BfieldMatDer2.zz'*BfieldMat +...
            BfieldMatDer.z'*BfieldMatDer.z;

        Hmat.xx = Hmat.xx' + Hmat.xx;
        Hmat.xy = Hmat.xy' + Hmat.xy;
        Hmat.xz = Hmat.xz' + Hmat.xz;

        Hmat.yy = Hmat.yy' + Hmat.yy;
        Hmat.yz = Hmat.yz' + Hmat.yz;

        Hmat.zz = Hmat.zz' + Hmat.zz;
    end
end
end

function [U, F, H] = evalU(Ivec, m, Umat, Fmat, Hmat)
%% evalUpol - evaluates U-operators for given Ivec
% The function evaluates potential energy, force and Hess matrix for given
% Ivec and coeficient alpha of polarizable or permanent dipole.
%
%   INPUTS:
%    Ivec: vector [N x 1]
%    m: dipole type ('pol', 'perm', 'Bfield') and value structure
%    Umat: potential energy operator [m x m]
%    Fmat: force operators structur(x, y, z)
%    Hmat: Hess matrix elements operators (xx, xy, xz, yy, yz, zz)
%
%   OUTPUTS:
%    U: potential energy
%    F: force [3 x 1]
%    H: Hess matrix [3 x 3]
%
%   SYNTAX
%
%    [U, F, H] = evalUpol(Ivec, alpha, Umat, Fmat, Hmat)
%
% Jakub Liska, FEE CTU in Prague, jakub.liska@fel.cvut.cz

% energy
U = real(Ivec'*Umat*Ivec);

if nargout > 1
    % force
    F = real([Ivec'*Fmat.x*Ivec;...
        Ivec'*Fmat.y*Ivec;...
        Ivec'*Fmat.z*Ivec]);

    if nargout > 2
        H = zeros(3); % Hessian matrix
        H(1,1) = Ivec'*Hmat.xx*Ivec;
        H(1,2) = Ivec'*Hmat.xy*Ivec;
        H(1,3) = Ivec'*Hmat.xz*Ivec;
        H(2,2) = Ivec'*Hmat.yy*Ivec;
        H(2,3) = Ivec'*Hmat.yz*Ivec;
        H(3,3) = Ivec'*Hmat.zz*Ivec;
    end
end
switch m.type
    case 'pol' % polarizable
        % energy
        U = - m.value/2*U;

        if nargout > 1
            % Force
            F = - m.value/2*F;

            if nargout > 2
                % Hess matrix
                H = - m.value/2*H;
            end
        end

    case 'perm' % permanent
        % Energy
        U = m.value*sqrt(U);
        if nargout > 1
            cnst = m.value^2/(2*U);
            % Force
            F = cnst*F;

            if nargout > 2
                % Hess matrix
                H(1,1) = cnst*H(1,1) - F(1)^2/U;
                H(1,2) = cnst*H(1,2) - F(1)*F(2)/U;
                H(1,3) = cnst*H(1,3) - F(1)*F(3)/U;
                H(2,2) = cnst*H(2,2) - F(2)^2/U;
                H(2,3) = cnst*H(2,3) - F(2)*F(3)/U;
                H(3,3) = cnst*H(3,3) - F(3)^2/U;
            end
        end
end

if nargout > 2
    H = real(H + H' - diag(diag(H)));
end
end