%% exMaxGvsDelta - example of use maxGvsDelta
% This stript provides example of use maxGvsDelta function to maximize gain for
% given structure with radiation matrix R0, lossy matrix Rmat directivity matrix
% U and dissipation factor.
%
% AToM package Fundamental Bounds, info@antennatoolbox.com
% (c) 2020, Jakub Liska, CTU in Prague, jakub.liska@antennatoolbox.com
% mcode

clc;
clear;
close all;

%% Mesh
Nx = 10;
Ny = 5;
PX = 2*ones(Ny, Nx);
[~, ~, mesh] = models.utilities.meshPublic.pixelGridToMesh(PX, 1);

% mesh = geometry.sphericalShell(38);
%% MoM matrices
ka = 0.5; % electrical size
Zs = 0.01; % surface impedance

% smallest circumscribing sphere
a = models.utilities.meshPublic.getCircumsphere(mesh.nodes);

f = ka*models.utilities.constants.c0()/(2*pi*a); % frequaency
[OP, ~] = models.utilities.matrixOperators.MoM2D.batch.evaluate(mesh, f, Zs,...
   'normalize', true, 'symmetrize', true, 'quadOrder', 5,...
   'requests', {'R0', 'Rmat', 'U'}, 'usegpu', true); % get matrix operators
OP.U = OP.U_hndl(f, 0, pi/2, 'phi');
%% compute adding constraint
% pareto borders
[optDelta, Ivec] = optimProblems.minDelta(OP);
GofOptQ = 8*pi*real((Ivec'*OP.U*Ivec)/(Ivec'*(OP.R0 + OP.Rmat)*Ivec));

[optG, Ivec] = optimProblems.maxG(OP);
DeltaOfOptG = real((Ivec'*OP.Rmat*Ivec)/(Ivec'*OP.R0*Ivec));

nParetoPoints = 20;

% initialization
nu = [0, 0];

normDeltaList = nan(1,nParetoPoints);
normGlist = nan(1,nParetoPoints);

normGlist(1) = GofOptQ/optG;
normGlist(2) = 1;

normDeltaList(1) = 1;
normDeltaList(2) = optDelta/DeltaOfOptG;

% pareto points
for iPoint = 3:nParetoPoints
   % distance between points
   dist = abs(normGlist(2:iPoint-1) - normGlist(1:iPoint-2)).*...
      abs(normDeltaList(2:iPoint-1) - normDeltaList(1:iPoint-2));
   [~, maxDist] = max(dist); % maximal distance
   
   % new point
   normDeltaList(iPoint) = mean(normDeltaList([maxDist, maxDist + 1]));
   [Gopt, ~, nu] = optimProblems.maxGvsDelta(OP,...
      optDelta/normDeltaList(iPoint), 'verbosity', 1, 'lambdaVec', nu);
   normGlist(iPoint) = Gopt/optG;
   
   [normDeltaList, ind] = sort(normDeltaList);
   normGlist = normGlist(ind);
end

hndlPareto = figure();
plot(normDeltaList, normGlist, 'o--');
xlabel('Delta_{min}/Delta');
ylabel('G/G_{max}');

%% compute convex
cList = linspace(0,1,nParetoPoints);
Glist = nan(1,nParetoPoints);
deltaList = nan(1,nParetoPoints);

normMat.U = OP.U/norm(OP.U);
normMat.R = OP.R0/norm(OP.R0);
normMat.RL = (OP.Rmat + OP.R0)/norm(OP.Rmat + OP.R0);

t = tic;
for ind = 1:nParetoPoints
   [~, ~, Ivec] = solvers.QCQP.minAstB1(...
      -cList(ind)*normMat.U - (1 - cList(ind))*normMat.R,...
      normMat.RL, 0);
   Glist(ind) = 8*pi*real((Ivec'*OP.U*Ivec)/(Ivec'*(OP.Rmat + OP.R0)*Ivec));
   deltaList(ind) = real((Ivec'*OP.Rmat*Ivec)/(Ivec'*OP.R0*Ivec));
end
t2 = toc(t);

figure(hndlPareto);
hold on;
plot(optDelta./deltaList, Glist/optG, 'x:');
hold off;