%% exMaxGvsDeltaSelfRes
% This script provides computation of Q-factor, dissipation factor pareto
% optimal points using convex cobination.
%
% AToM package Fundamental Bounds, info@antennatoolbox.com
% (c) 2020, Jakub Liska, CTU in Prague, jakub.liska@antennatoolbox.com
% mcode

clc;
clear;
close all;

nParetoPoints = 50;
verbosity = 1;

%% Mesh
Nx = 6;
Ny = 3;
PX = 2*ones(Ny, Nx);
[~, ~, mesh] = models.utilities.meshPublic.pixelGridToMesh(PX, 1);

% mesh = geometry.sphericalShell(38);
%% MoM matrices
ka = 0.5; % electrical size
Zs = 0.01; % surface impedance

% smallest circumscribing sphere
a = models.utilities.meshPublic.getCircumsphere(mesh.nodes);

f = ka*models.utilities.constants.c0()/(2*pi*a); % frequaency
[OP, ~] = models.utilities.matrixOperators.MoM2D.batch.evaluate(mesh, f, Zs,...
   'normalize', true, 'symmetrize', true, 'quadOrder', 5,...
   'requests', {'R0', 'Rmat', 'U', 'X0', 'Xmat'}, 'usegpu', true); % get matrix operators
OP.U = OP.U_hndl(f, 0, pi/2, 'phi');

%% compute convex
cList = linspace(0,1,nParetoPoints).^(1/2);
Glist = nan(1,nParetoPoints);
deltaList = nan(1,nParetoPoints);

normMat.U = OP.U/norm(OP.U);
normMat.R = OP.R0/norm(OP.R0);
normMat.RL = (OP.Rmat + OP.R0)/norm(OP.Rmat + OP.R0);
normMat.X = (OP.Xmat + OP.X0)/norm(OP.Xmat + OP.X0);

lambdaVec = zeros(1,2);

for ind = 1:nParetoPoints
   [~, ~, Ivec, lambdaVec] = solvers.QCQP.minAstBn(...
      - cList(ind)*normMat.R - (1 - cList(ind))*normMat.U,...
      lambdaVec, verbosity,...
      normMat.RL, {}, - 1,...
      normMat.X, {}, 0);
   Glist(ind) = 8*pi*real((Ivec'*OP.U*Ivec)/(Ivec'*(OP.Rmat + OP.R0)*Ivec));
   deltaList(ind) = real((Ivec'*OP.Rmat*Ivec)/(Ivec'*OP.R0*Ivec));
end

deltaMat = OP.Rmat - deltaList(end)*OP.R0;
[~, ~, Ivec] = solvers.QCQP.minAstBn(...
   - normMat.U,...
   zeros(1,3), verbosity,...
   normMat.RL, {}, - 1,...
   normMat.X, {}, 0,...
   deltaMat/norm(deltaMat), {}, 0);

Glist(end) = 8*pi*real((Ivec'*OP.U*Ivec)/(Ivec'*(OP.Rmat + OP.R0)*Ivec));

h = figure();
plot(min(deltaList)./deltaList, Glist/max(Glist), 'x:');
xlabel('Delta_{min}/Delta');
ylabel('G/G_{max}');