%% exMaxPaCUZpowerConstMoM2D - example of the use of minMaxPaPsPeZpowerConst
% This example shows an evaluation of maximum absorption constrained by
% complex power conservation, using controllable region and far-field 
% constraints. The absorbed power in the entire structure is optimized. 
% The optimization assumes a user defined excitation via vector V.
%
% Included in AToM, info@antennatoolbox.com
% (c) 2021, Lukas Jelinek, CTU in Prague, lukas.jelinek@antennatoolbox.com
% mcode

clear;
close all;
clc;

%% dimensions
Wpatch = 17e-3; % x-width of patch
Lpatch = Wpatch; % y-width of patch 

Wground = 60e-3; % x-width of ground
Lground = 60e-3; % y-width of ground

height = 3*1e-3; % height of patch above ground

%% Create and plot mesh - Pixellized structure of Nx x Ny cells

% bottom patch
NxGround = 11; % number of pixels in x-direction
NyGround = 11; % number of pixels in y-direction
PX  = 1*ones(NyGround, NxGround);
[nodes, connectivityList] = ...
   models.utilities.meshPublic.pixelGridToOrthoMesh(PX, 1);

% top patch
NxPatch = 10; % number of pixels in x-direction
NyPatch = 10;  % number of pixels in y-direction
PX  = 1*ones(NyPatch, NxPatch);
[nodesPatch, connectivityListPatch] = ...
   models.utilities.meshPublic.pixelGridToOrthoMesh(PX, 1);

% scale
nodes(:,1) = nodes(:,1) / NxGround * Lground;
nodes(:,2) = nodes(:,2) / NyGround * Wground;
nodesPatch(:,1) = nodesPatch(:,1) / NxPatch * Wpatch;
nodesPatch(:,2) = nodesPatch(:,2) / NyPatch * Lpatch;

% shift top patch
nodesPatch = models.utilities.meshPublic.translateMesh(...
    nodesPatch, [0,0,height]);

% join meshes
[nodes, connectivityList] = ...
    models.utilities.meshPublic.uniteMeshes(nodes, ...
   connectivityList, nodesPatch, connectivityListPatch);

% create mesh structure
mesh = models.utilities.meshPublic.getMeshData2D(nodes, connectivityList);

%% Set frequency range and initialize variables
Nka = 30; % number of frequency points
kaList = mesh.normDistanceA/Wpatch*0.953*...
    linspace(1 - 0.01,1 + 0.01,Nka); % electric size

fList = nan(1,Nka); % frequency
ZsList = nan(1,Nka); % surface resistance
PaOptList = nan(1,Nka); % optimized absorbed power

%% Excitation
% planewave
planeWaveData.propagationVector = [0 0 -1];
planeWaveData.initElectricField = [0 1 0];
planeWaveData.axialRatio = Inf;
planeWaveData.direction = 'right';

%% Setting controllable region - rectangular prism
% prepare basis functions
basisFunc = models.solvers.MoM2D.basisFcns.getBasisFcns(mesh);

userData.polarity = 'uncontrollable'; % size data define (un)controllable
userData.type = 'rectPrism';
userData.dx = 2*Lground; % x-size of selected region
userData.dy = 2*Lground; % y-size of selected region
userData.dz = height; % z-size of selected region
userData.centre = [0,0,0]; % center of selected region

[NBF, NBFc, NBFu, BFtoBFc, BFtoBFu] = ...
   controllableRegion.defineControllableRegion (mesh, basisFunc, userData);

%% plot mesh

options.showEdges = true;
options.showTriangles = true;
hndlMesh = results.plotMesh(mesh, 'options', options);
hndlMesh.triangles.FaceColor = 'flat';
hndlMesh.triangles.FaceVertexCData = ...
    repmat([255,255,153]/255,[mesh.nTriangles,1]);
view(-36,33);

% make edges in controlable region blue
% set( hndlMesh.edges(basisFunc.data(BFtoBFc,3)) , 'Color', [0 0 1]);

% make triangles in controlable region blue
triaInd = unique([basisFunc.data(BFtoBFc,2);basisFunc.data(BFtoBFc,4)]);
hndlMesh.triangles.FaceVertexCData(triaInd,:) = ...
    repmat([102,192,237]/255,[size(triaInd,1),1]);

% triange opacity
hndlMesh.triangles.FaceAlpha = 0.7;

%% Solver settings
MoMVerosity    = 0; % verbosity of MoM
OptimVerbosity = 1; % verbosity of optimization
normalize = true; % true / false  - normalization of MoM quantities
quadOrder = 5; % order of quadrature

%% frequency sweep

% initial Lagrange's multipliers
lambdaInitList = [- 1; 0];

for ika = Nka:-1:1

   ka = kaList(1,ika);
   k0 = ka/mesh.normDistanceA;
   lambda0 = 2*pi/k0;
   f0 = models.utilities.constants.c0/lambda0;
   fList(1,ika) = f0;
   
   ZsList(1,ika) = 0.01;
   
   % evaluate MoM matrices
   [OP, ~] = models.utilities.matrixOperators.MoM2D.batch.evaluate(...
       mesh, f0, ZsList(1,ika), 'quadOrder', quadOrder, 'verbosity', ...
       MoMVerosity, 'normalize', normalize, 'symmetrize', true, ...
       'usegpu', true, 'requests', {'R0','Rmat', 'X0','Xmat', ...
       'U','Fph', 'Fth'});
   
   % define radiation intensity direction, component and frequency
   OP.theta = 0;
   OP.phi = pi/2;
   OP.component = 'total';
   OP.f0 = f0;
   OP.U = OP.U_hndl(OP.f0, OP.theta, OP.phi, OP.component);
   
   %% Excitation
   % planewave
   V = models.solvers.MoM2D.excitation.planeWave(mesh, basisFunc, ...
      planeWaveData, k0, quadOrder);
   
   if normalize
      %  normalized --> V [Volt]
      V = V./mesh.triangleEdgeLengths(OP.BF.data(:,3),1);
   end
   
   %% affine constraints
   [~, ~, ~, ~, ~, Vu] = controllableRegion.decompHndls(BFtoBFc, BFtoBFu);
   
   % affine constraint on C/U region
   OP.Aaff = Vu(OP.R0 + OP.Rmat + 1i*OP.X0 + 1i*OP.Xmat);
   OP.aAff = - Vu(V);
   
   % affine constraint on far-field
   OP.Aaff = [OP.Aaff; ...
              OP.Fth_hndl(OP.f0, OP.theta, OP.phi);...
              OP.Fph_hndl(OP.f0, OP.theta, OP.phi)];
   OP.aAff = [OP.aAff;0;0];    
   
   %% optimization

   % optimization
   [PaOptList(1,ika), ~, ~, IOpt, lambdaInitList, tocTime] = ...
    optimProblems.minMaxPaPsPeZpowerConst(OP, V, 'max', 'Pa', ...
    'lambdaVec', lambdaInitList, 'verbosity', OptimVerbosity);
  
   UOptList(1, ika) = (IOpt'*OP.U_hndl(OP.f0, OP.theta, OP.phi, ...
   OP.component)*IOpt);
   
   if normalize
      % remove normalization --> I[A/m]
      IOpt = IOpt./mesh.triangleEdgeLengths(OP.BF.data(:,3),1);
   end
   
disp(['frequency countdown = ',num2str(Nka - ika)])
end

%% Plot results
kWpatch = kaList/mesh.normDistanceA*Wpatch;
S0   = norm(planeWaveData.initElectricField)^2 / ...
   (2*models.utilities.constants.Z0); % impinging power flux
Area = Lground*Wground;

figure;
semilogy(kWpatch,real(PaOptList/(S0*Area)));
grid on;
xlabel('kW');
ylabel('P_a / ( S_0 Area)');


%% Plot current
h = results.plotCurrent(mesh, 'basisFcns', basisFunc, 'Ivec', IOpt, ...
    'part', 'abs', 'scale', 'logarithmic', 'arrowScale', 'proportional');
view(-36,33);

%% Plot radiation pattern
theta = linspace(0, pi, 200);
phi = linspace(0, 2*pi, 200);

farfield = results.calculateFarField(mesh, f0, 'theta', theta, ...
   'phi', phi, 'basisFcns', basisFunc, 'iVec', IOpt);
handleFF = results.plotFarField('farField', abs(farfield.D), ...
   'theta', theta, 'phi', phi);
view(-36,33);