%% exMaxPaPsPeZpowerConstMoM2D - example of the use of minMaxPaPsPeZpowerConst
% This example shows an evaluation of maximum absorption constrained by
% complex power conservation. The optimization assumes a user defined 
% excitation via vector V.
%
% Included in AToM, info@antennatoolbox.com
% (c) 2020, Lukas Jelinek, CTU in Prague, lukas.jelinek@antennatoolbox.com
% mcode

clear;
close all;
clc;

%% Create and plot mesh
% Pixellized structure of Nx x Ny cells
Nx = 10; 
Ny = 5;
PX  = 1*ones(Ny, Nx); % metal
PX([1,2,3],[4,5]) = 0; % vacuum
[nodes, connectivityList] = ...
    models.utilities.meshPublic.pixelGridToOrthoMesh(PX, 1);

% scale mesh
nodes = nodes / Nx;

% create mesh structure
mesh = models.utilities.meshPublic.getMeshData2D(nodes, connectivityList);

% plot mesh
options.showEdges = true;
options.showTriangles = true;
hndlMesh = results.plotMesh(mesh, 'options', options);
view(0,90)

% prepare basis functions
basisFunc = models.solvers.MoM2D.basisFcns.getBasisFcns(mesh);

%% Set frequency range and initialize variables
Nka = 100;
% kaList = 10.^linspace(-2,log10(3),Nka); % electric size
kaList = linspace(0.3,3,Nka); % electric size

% Nka = 1;
% kaList = 2; % electric size

fList = nan(1,Nka); % frequency
ZsList = nan(1,Nka); % surface resistance

PaOptList = nan(1,Nka); % optimized absorbed power
PaFedList = nan(1,Nka); % optimized absorbed power

%% Excitation
% planewave
planeWaveData.propagationVector = [0 0 1];
planeWaveData.initElectricField = [0 1 0];
planeWaveData.axialRatio = Inf;
planeWaveData.direction = 'right';

%% Solver settings
MoMVerosity    = 1;
OptimVerbosity = 1;
normalize = true; % true / false  - normalization of MoM quantities
quadOrder = 5;

%% Solve MoM
lambdaInitList = [-1.204501888630410e+00; 4.301072731350373e-05];
for ika = Nka:-1:1
    ka = kaList(1,ika);
    k0 = ka/mesh.normDistanceA;
    lambda0 = 2*pi/k0;
    f0 = models.utilities.constants.c0/lambda0;
    fList(1,ika) = f0;
    
    ZsList(1,ika) = 0.01;
    
    [OP, ~] = models.utilities.matrixOperators.MoM2D.batch.evaluate(mesh, f0,...
        ZsList(1,ika), 'quadOrder', quadOrder, 'verbosity', MoMVerosity,...
        'normalize', normalize, 'symmetrize', true, 'requests', ...
        {'R0','Rmat', 'X0','Xmat'});
     
    %% Excitation
    % planewave
    V = models.solvers.MoM2D.excitation.planeWave(mesh, basisFunc, ...
        planeWaveData, k0, quadOrder);
    if normalize
        %  normalized --> V [Volt]
        V = V./mesh.triangleEdgeLengths(OP.BF.data(:,3),1);
    end

    %% True absorption
    iFed = mldivide(OP.R0 + OP.Rmat + 1i*OP.X0 + 1i*OP.Xmat, V);
    PaFedList(1, ika) = 0.5*iFed'*OP.Rmat*iFed;

   %% affine constraints
   OP.Aaff = [];
   OP.aAff = [];
   
   %% optimization

   % optimization
   [PaOptList(1,ika), ~, ~, IOpt, lambdaInitList, tocTime] = ...
    optimProblems.minMaxPaPsPeZpowerConst(OP, V, 'max', 'Pa', ...
    'lambdaVec', lambdaInitList, 'verbosity', OptimVerbosity);

    if normalize
        % remove normalization --> I[A/m]
        IOpt = IOpt./mesh.triangleEdgeLengths(OP.BF.data(:,3),1);
    end
    
    disp(num2str(Nka - ika))
end

%% Plot results
S0   = norm(planeWaveData.initElectricField)^2 / ...
       (2*models.utilities.constants.Z0);
Area = sum(mesh.triangleAreas); %pi*mesh.normDistanceA^2;

figure;
semilogy(kaList,real(PaFedList/(S0*Area)));
hold on
semilogy(kaList,real(PaOptList/(S0*Area)));
grid on;
xlabel('ka');
ylabel('Pa / S0 / A');
legend('true','optimized');

% electric size of the maximum absorbtion in original structure
% kaList(find( real(PaOptList) == max(real(PaOptList))))

%% Plot current
h = results.plotCurrent(mesh, 'basisFcns', basisFunc, 'Ivec', IOpt, 'part', 'abs', 'scale', 'logarithmic', ...
     'arrowScale', 'proportional');
view(0,90)

%% Plot radiation pattern
theta = linspace(0, pi, 200);
phi = linspace(0, 2*pi, 200);

farfield = results.calculateFarField(mesh, f0, 'theta', theta, ...
    'phi', phi, 'basisFcns', basisFunc, 'iVec', IOpt);
handleFF = results.plotFarField('farField', abs(farfield.D), ...
    'theta', theta, 'phi', phi);
view(-36,33);