% clc, clear, close all;

nParetoPoints = [10, 5]; % [Q-sweep, G-delta]
verbosity = 1;

%% Mesh
Nx = 6;
Ny = 3;
PX = 2*ones(Ny, Nx);
[~, ~, mesh] = models.utilities.meshPublic.pixelGridToMesh(PX, 1);
%% MoM matrices
ka = 0.5; % electrical size
Zs = 0.01; % surface impedance

% smallest circumscribing sphere
a = models.utilities.meshPublic.getCircumsphere(mesh.nodes);

f = ka*models.utilities.constants.c0()/(2*pi*a); % frequaency
[OP, ~] = models.utilities.matrixOperators.MoM2D.batch.evaluate(mesh, f, Zs,...
   'normalize', true, 'symmetrize', true, 'quadOrder', 7,...
   'requests', {'R0', 'omW', 'Rmat', 'U', 'X0', 'Xmat'}, 'usegpu', true); % get matrix operators
OP.U = OP.U_hndl(f, pi/2, pi/2, 'phi'); % endfire
% OP.U = OP.U_hndl(f, 0, 0, 'theta'); % broadside

normMat_fun = @(M) M/norm(M);

normMat.R0 = normMat_fun(OP.R0);
normMat.W = normMat_fun(OP.omW);
normMat.X = normMat_fun(OP.X0 + OP.Xmat);
normMat.Rmat = normMat_fun(OP.Rmat);
normMat.U = normMat_fun(OP.U);
normMat.RL = normMat_fun(OP.R0 + OP.Rmat);

G_fun = @(x) 8*pi*real((x'*OP.U*x)/(x'*(OP.Rmat + OP.R0)*x));
Q_fun = @(x) 0.5*real(((x'*OP.omW*x) + abs(x'*OP.X0*x))/(x'*OP.R0*x));
delta_fun = @(x) real((x'*OP.Rmat*x)/(x'*OP.R0*x));

Q_constraintMat = @(Q0) normMat_fun(0.5*OP.omW - Q0*OP.R0);
G_constraintMat = @(G0) normMat_fun(8*pi*OP.U - G0*(OP.R0 + OP.Rmat));
delta_constraintMat = @(d0) normMat_fun(OP.Rmat - d0*OP.R0);

%% pareto
Qlist = nan(nParetoPoints);
deltaList = nan(nParetoPoints);
Glist = nan(nParetoPoints);

%% G-delta pareto
alphaList = linspace(0, 1, nParetoPoints(2));
lambdaVec = zeros(1,2);

for ind = 1:nParetoPoints(2)
   alpha = alphaList(ind);
   [~, ~, Ivec, lambdaVec] = solvers.QCQP.minAstBn(...
      - (1 - alpha)*normMat.R0 - alpha*normMat.U,...
      lambdaVec, verbosity,...
      normMat.RL, {}, - 1,...
      normMat.X, {}, 0);
   
   Qlist(1,ind) = Q_fun(Ivec);
   deltaList(1,ind) = delta_fun(Ivec);
   Glist(1,ind) = G_fun(Ivec);
end

[~, ~, Ivec] = solvers.QCQP.minAstBn(...
   - normMat.U,...
   zeros(1,3), verbosity,...
   normMat.RL, {}, - 1,...
   normMat.X, {}, 0,...
   normMat_fun(OP.Rmat - deltaList(1,1)*OP.R0), {}, 0);

Qlist(1,1) = Q_fun(Ivec);
deltaList(1,1) = delta_fun(Ivec);
Glist(1,1) = G_fun(Ivec);

%% min Q
Qmin = optimProblems.minQselfRes(OP, 'verbosity', verbosity);
Qmax =max(Qlist(1,:));
QminGdelta = min(Qlist(1,:));

%% sweep Q
cExp = -2;
Qswp = linspace(Qmax.^(1/cExp), Qmin.^(1/cExp), nParetoPoints(1)).^cExp;
Qlist(2:end,:) = repmat(Qswp(2:end).', 1, nParetoPoints(2));

lambdaVec2 = zeros(1,4);
for indQ = 2:nParetoPoints(1)
   lambdaVec = zeros(1,3);
   for ind = 1:nParetoPoints(2)
      if Qlist(indQ,ind) < Qlist(1,ind)
         alpha = alphaList(ind);
         [~, ~, Ivec, lambdaVec] = solvers.QCQP.minAstBn(...
            - (1 - alpha)*normMat.R0 - alpha*normMat.U,...
            lambdaVec, verbosity,...
            normMat.RL, {}, - 1,...
            normMat.X, {}, 0,...
            normMat_fun(Q_constraintMat(Qlist(indQ,ind))), {}, 0);
         
         if ind == 1
            deltaList(indQ,1) = delta_fun(Ivec);
            
            [~, ~, Ivec, lambdaVec2] = solvers.QCQP.minAstBn(...
               - normMat.U,...
               lambdaVec2, verbosity,...
               normMat.RL, {}, - 1,...
               normMat.X, {}, 0,...
               normMat_fun(Q_constraintMat(Qlist(indQ,1))), {}, 0,...
               normMat_fun(delta_constraintMat(deltaList(indQ,1))));
         end
         
         Qlist(indQ,ind) = Q_fun(Ivec);
         deltaList(indQ,ind) = delta_fun(Ivec);
         Glist(indQ,ind) = G_fun(Ivec);
      else
         Qlist(indQ,ind) = Qlist(1,ind);
         deltaList(indQ,ind) = deltaList(1,ind);
         Glist(indQ,ind) = Glist(1,ind);
      end
   end
end

%% figure
figure();
surf(Qmin./Qlist, deltaList(1,1)./deltaList, Glist/Glist(1,end));
xlabel('Q_{min}/Q');
ylabel('\delta_{min}/\delta');
zlabel('G/G_{max}');