%% exMinQselfResVsDelta - example of use minQselfResVsDelta
% This stript provides example of use minQselfResVsDelta function to minimize
% selfresonant Q for given structure with radiation matrix R0, reakctance matrix
% X0, stored energy matrix omW and Delta.
%
% AToM package Fundamental Bounds, info@antennatoolbox.com
% (c) 2021, Jakub Liska, CTU in Prague, jakub.liska@antennatoolbox.com
% mcode

clc;
clear;
close all;

nParetoPoints = 100;
charModAcceleration = true;

%% Mesh
Nx = 20;
Ny = 10;
PX = 2*ones(Ny, Nx);
[~, ~, mesh] = models.utilities.meshPublic.pixelGridToOrthoMesh(PX, 1);

% mesh = geometry.sphericalShell(38);
%% MoM matrices
ka = 0.5; % electrical size
Zs = 0.01; % surface impedance

% smallest circumscribing sphere
a = models.utilities.meshPublic.getCircumsphere(mesh.nodes);

f = ka*models.utilities.constants.c0()/(2*pi*a); % frequaency
[OP, ~] = models.utilities.matrixOperators.MoM2D.batch.evaluate(mesh, f, Zs,...
    'normalize', true, 'symmetrize', true, 'quadOrder', 7,...
    'requests', {'R0', 'X0', 'omW', 'Rmat', 'Xmat'}, 'usegpu', true); % get matrix operators

%% char mod accelareation
if charModAcceleration
    [Vm, lam] = eig(OP.R0, OP.X0, 'vector');
    mModes = sum(abs(lam) > 1e-15);
    V = normalize(Vm(:, 1:mModes),1, 'norm');
    
    OP.R0 = V'*OP.R0*V;
    OP.omW = V'*OP.omW*V;
    OP.X0 = V'*OP.X0*V;
    OP.Xmat = V'*OP.Xmat*V;
    OP.Rmat = V'*OP.Rmat*V;
end

%% clasical parreto
nParetoPoints2 = 80;
p = tic;
alphaList = nan(1,nParetoPoints2);
alphaList(1:3) = [0, 0.5, 1];
Qlist2 = nan(1,nParetoPoints2);
DeltaList2 = nan(1,nParetoPoints2);
lambdaVec = zeros(1,2);
verbosity = 0;

norm_mat = @(M) M/norm(M);
normMat.R0 = norm_mat(OP.R0);
normMat.X = norm_mat(OP.X0 + OP.Xmat);

for ind = 1:nParetoPoints2
    alpha = alphaList(ind);
    normMat.P = norm_mat(alpha*0.5*OP.omW + (1 - alpha)*OP.Rmat);
    [~, ~, Ivec, lambdaVec] = solvers.QCQP.minAstBn( - normMat.R0,...
        lambdaVec, verbosity,...
        normMat.P, {}, - 1,...
        normMat.X, {}, 0);
    
    Qlist2(ind) = 0.5*real(((Ivec'*OP.omW*Ivec) + abs(Ivec'*OP.X0*Ivec))/...
        (Ivec'*OP.R0*Ivec));
    DeltaList2(ind) = real((Ivec'*OP.Rmat*Ivec)/(Ivec'*OP.R0*Ivec));
    
    if ind > 2
        [alphaList, sIndex] = sort(alphaList);
        Qlist2 = Qlist2(sIndex);
        DeltaList2 = DeltaList2(sIndex);
        
        if ind < nParetoPoints2
            dist = abs(DeltaList2(2:ind-1) - DeltaList2(1:ind-2)).*...
                abs(Qlist2(2:ind-1) - Qlist2(1:ind-2));
            [~, maxDist] = max(dist);
            alphaList(ind+1) = mean(alphaList([maxDist, maxDist + 1]));
        end
    end
end

optQselfRes = min(Qlist2);
optDeltaSelfRes = min(DeltaList2);

nQ = Qlist2/optQselfRes;
nDelta = DeltaList2/optDeltaSelfRes;

toc(p)

%% compute by adding constraint
% pareto borders
cp = tic;

% pareto points
for iPoint = nParetoPoints2+1:nParetoPoints
    % distance between points
    dist = abs(nDelta(2:iPoint-1) - nDelta(1:iPoint-2)).*...
        abs(nQ(2:iPoint-1) - nQ(1:iPoint-2));
    [~, maxDist] = max(dist); % maximal distance
    
    % new point
    nDelta(iPoint) = mean(nDelta([maxDist, maxDist + 1]));
    Qopt = optimProblems.minQselfResVsDelta(OP,...
        optDeltaSelfRes*nDelta(iPoint),'verbosity', 0);
    nQ(iPoint) = Qopt/optQselfRes;
    
    [nDelta, ind] = sort(nDelta);
    nQ = nQ(ind);
end
toc(cp)

fHndl = figure();
plot(nDelta, nQ, 'o-');
xlabel('\delta/\delta_{min}');
ylabel('Q/Q_{min}');

data = [nDelta', nQ'];
save('paretoQdelta.tsv', 'data', '-ascii');