%% exMinQselfResZpowerConst - example of use minQselfResZpowerConst
% This example shows an evaluation of optimal Q-factor constrained by 
% complex power conservation using function 
% "optimProblems.minQselfResZpowerConst". The optimization assumes a 
% user defined excitation via vector V.
% 
% Included in AToM, info@antennatoolbox.com
% (c) 2019, Lukas Jelinek, CTU in Prague, lukas.jelinek@antennatoolbox.com
% mcode

clear;
close all;
clc;

%% create and plot mesh
% Pixellized rectangle Nx x Ny cells
Nx = 20; 
Ny = 10;
PX   = 1*ones(Ny, Nx);
% PX([3,4],[5,6]) = 0;
[nodes, connectivityList] = ...
    models.utilities.meshPublic.pixelGridToOrthoMesh(PX, 1);

% scale mesh
nodes = nodes / Nx;

% create mesh structure
mesh = models.utilities.meshPublic.getMeshData2D(nodes, connectivityList);

%% Optimization settings:
ka = 1/2; % electric size

ZsList = 0.01; % surface impedance
MoMVerosity = 2;
OptimVerbosity = 1;
normalize = true; % true / false  - normalization of MoM quantities
quadOrder = 1;

%% MoM matrices
a  = models.utilities.meshPublic.getCircumsphere(mesh.nodes);
k0 = ka/a;
f0 = ka*models.utilities.constants.c0/(2*pi*a);

[OP, ~] = models.utilities.matrixOperators.MoM2D.batch.evaluate(mesh, f0, ...
   ZsList, 'quadOrder', quadOrder ,'verbosity', MoMVerosity, ...
   'normalize', normalize, 'symmetrize', true, 'requests', ...
   {'R0','Rmat', 'X0','Xmat', 'omW'});

%% Excitation - delta gap

% find edge for delta gap
feederPosition = [0, 0.24, 0];
[~, nearestEdgePosition] = min(models.utilities.vectorOperations.rowNorm(...
    mesh.triangleEdgeCenters - feederPosition));
fedBF = find(OP.BF.data(:,3) == nearestEdgePosition);

% delta gap
V = zeros(OP.BF.nUnknowns, 1);
V(fedBF, 1) = 0.016;

if normalize
    %  normalized --> V [Volt]
    V = V./mesh.triangleEdgeLengths(OP.BF.data(:,3),1);
end

%% plot mesh

options.showEdges = true;
options.showTriangles = true;
hndlMesh = results.plotMesh(mesh, 'options', options);

% make the fed edge red
set(hndlMesh.edges(nearestEdgePosition) , 'Color', [1 0 0]);

%% Optimization
[QOpt, Qext, IOpt, lambdaVec, tocTimeZpower] = ...
    optimProblems.minQselfResZpowerConst ...
   (OP, V, 'lambdaVec', ...
   [-0.01;0;-0.01;0], 'verbosity', OptimVerbosity );

if normalize
    % remove normalization --> I[A/m]
    IOpt = IOpt./mesh.triangleEdgeLengths(OP.BF.data(:,3),1);
end

disp(['Duration of evaluation: ', num2str(tocTimeZpower), ' s', ...
    ' , Optimal Q-factor with Z power constraint = ', ...
    num2str(real(QOpt)) ]);

%% Plot optimal current
h = results.plotCurrent(mesh, 'basisFcns', OP.BF, 'Ivec', IOpt, ...
    'part', 'abs', 'scale', 'logarithmic', ...
     'arrowScale', 'proportional');

%% Plot radiation pattern
theta = linspace(0, pi, 200);
phi   = linspace(0, 2*pi, 200);

farfield = results.calculateFarField(mesh, f0, 'theta', theta, ...
   'phi', phi, 'basisFcns', OP.BF, 'iVec', IOpt);
handleFF = results.plotFarField('farField', abs(farfield.D), ...
   'theta', theta, 'phi', phi);

view(-32, 16);