%% exMaxQvsDeltaVsDswpSelfRes
% This script provides computation of Q-factor, dissipation factor and
% directivity pareto optimal points using convex cobination Q-delta and D
% sweep.
%
% AToM package Fundamental Bounds, info@antennatoolbox.com
% (c) 2021, Jakub Liska, CTU in Prague, jakub.liska@antennatoolbox.com
% mcode

clc, clear, close all;

nParetoPoints = [40, 20]; % [D-sweep, Q-delta]
verbosity = 1;
charModAcceleration = true;
direction = 1; % 0: broadside, 1: endfire

%% Mesh
Nx = 20;
Ny = 10;
PX = 2*ones(Ny, Nx);
[~, ~, mesh] = models.utilities.meshPublic.pixelGridToOrthoMesh(PX, 1);

% mesh = geometry.sphericalShell(110);
%% MoM matrices
ka = 0.5; % electrical size
Zs = 0.01; % surface impedance

% smallest circumscribing sphere
a = models.utilities.meshPublic.getCircumsphere(mesh.nodes);

f = ka*models.utilities.constants.c0()/(2*pi*a); % frequaency
[OP, ~] = models.utilities.matrixOperators.MoM2D.batch.evaluate(mesh, f, Zs,...
    'normalize', true, 'symmetrize', true, 'quadOrder', 7,...
    'requests', {'R0', 'omW', 'Rmat', 'U', 'X0', 'Xmat', 'Fph', 'Fth', 'S1'},...
    'usegpu', true); % get matrix operators
if direction == 0
    OP.F = OP.Fth_hndl(f, 0, 0); % broaside farfield
    OP.U = OP.U_hndl(f, 0, 0, 'theta'); % broadside
    OP.U2 = OP.U_hndl(f, pi, 0, 'theta'); % broadside2
else
    OP.F = OP.Fph_hndl(f, pi/2, 0); % endfire farfield
    OP.U = OP.U_hndl(f, pi/2, 0, 'phi'); % endfire
    OP.U2 = OP.U_hndl(f, pi/2, pi, 'phi'); % endfire2
end

%% char mod accelareation
if charModAcceleration
    [Vm, lam] = eig(OP.X0, OP.R0, 'vector');
    modes = abs(lam) < 1e15;
    
    [~, ~, Vf] = svd(OP.F, 'econ');
    
    V = [normalize(Vm(:,modes),1, 'norm'), Vf];
    if sum(modes)+1 ~= rank(V)
        figure(); bar3(abs(V'*V));
        warning('Matrix V has dependent columns!');
    end
    
    OP.R0 = V'*OP.R0*V;
    OP.omW = V'*OP.omW*V;
    OP.X0 = V'*OP.X0*V;
    OP.Xmat = V'*OP.Xmat*V;
    OP.Rmat = V'*OP.Rmat*V;
    OP.U = V'*OP.U*V;
    OP.U2 = V'*OP.U2*V;
end

%% prepare matrix and functions
normMat_fun = @(M) M/norm(M);

normMat.R0 = normMat_fun(OP.R0);
normMat.W = normMat_fun(OP.omW);
normMat.X = normMat_fun(OP.X0 + OP.Xmat);
normMat.Rmat = normMat_fun(OP.Rmat);
normMat.U = normMat_fun(OP.U);

D_fun = @(x) 8*pi*real((x'*OP.U*x)/(x'*OP.R0*x));
D_fun2 = @(x) 8*pi*real((x'*OP.U2*x)/(x'*OP.R0*x));
Q_fun = @(x) 0.5*real(((x'*OP.omW*x) + abs(x'*OP.X0*x))/(x'*OP.R0*x));
delta_fun = @(x) real((x'*OP.Rmat*x)/(x'*OP.R0*x));

D_constraintMat = @(D0) normMat_fun(8*pi*OP.U - D0*OP.R0);

%% G pareto initialization
Qlist = nan(nParetoPoints);
deltaList = nan(nParetoPoints);
Dlist = nan(nParetoPoints);

if direction == 0
    eAlpha = 3; % broadside
else
    eAlpha = 7; % endfire
end
alphaList = linspace(0, 1, nParetoPoints(2)).^eAlpha;

%% no D
lambdaVec = zeros(1,2);
ang = [pi/2, pi];
for ind = 1:nParetoPoints(2)
    alpha = alphaList(ind);
    matP = normMat_fun(alpha*OP.omW + (1 - alpha)*OP.Rmat);
    matP = 0.5*(matP + matP');
    
    [~, ~, Ivec, lambdaVec, dg] = solvers.QCQP.minAstBn( - normMat.R0,...
        lambdaVec, verbosity,...
        matP, {}, - 1,...
        normMat.X, {}, 0);
    
    if dg
        err = [Ivec'*matP*Ivec - 1;...
            Ivec'*normMat.X*Ivec];
        disp(err);
    end
    
    Qlist(1,ind) = Q_fun(Ivec);
    deltaList(1,ind) = delta_fun(Ivec);
    Dlist(1,ind) = max([D_fun(Ivec), D_fun2(Ivec)]);
    %
    %     [ang, ~] = fminsearch(@(ang) -abs(OP.Fph_hndl(f, ang(1),ang(2))*...
    %         Ivec), ang);
    %     Dlist(1,ind) = real(abs(OP.Fph_hndl(f, ang(1),ang(2))*...
    %         Ivec)^2/(Ivec'*OP.R0*Ivec));
end

%% D
Dmin = min(Dlist(1,:));
if direction == 0
    Dmax = 1.3*Dmin; % broadside
else
    Dmax = 20*Dmin; % endfire
end
ec = 1;

Dswp = Dmin + (Dmax - Dmin)*linspace(0, 1, nParetoPoints(1)).^ec;
Dlist(2:end,:) = repmat(Dswp(2:end).', 1, nParetoPoints(2));

%% D sweep
for indD = 2:nParetoPoints(1)
    lambdaVec = zeros(1,3);
    for ind = 1:nParetoPoints(2)
        if Dlist(indD,ind) > Dlist(1,ind)
            alpha = alphaList(ind);
            
            matP = normMat_fun(alpha*OP.omW + (1 - alpha)*OP.Rmat);
            matP = 0.5*(matP + matP');
            matD = normMat_fun(D_constraintMat(Dlist(indD,ind)));
            
            try
                
                [~, ~, Ivec, lambdaVec, dg] = solvers.QCQP.minAstBn( - normMat.R0,...
                    lambdaVec, verbosity,...
                    matP, {}, - 1,...
                    normMat.X, {}, 0,...
                    matD, {}, 0);
            catch msg
                warning('Divergence of dual problem');
                Qlist(indD,ind) = Qlist(indD-1,ind);
                deltaList(indD,ind) = deltaList(indD-1,ind);
                Dlist(indD,ind) = Dlist(indD-1,ind);
            end
            
            if dg || isempty(Ivec)
                err = [Ivec'*matP*Ivec - 1;...
                    Ivec'*normMat.X*Ivec;...
                    Ivec'*matD*Ivec];
                disp(err);
                
                Qlist(indD,ind) = Qlist(indD-1,ind);
                deltaList(indD,ind) = deltaList(indD-1,ind);
                Dlist(indD,ind) = Dlist(indD-1,ind);
            else
                
                Qlist(indD,ind) = Q_fun(Ivec);
                deltaList(indD,ind) = delta_fun(Ivec);
                Dlist(indD,ind) = D_fun(Ivec);
            end
        else
            Qlist(indD,ind) = Qlist(1,ind);
            deltaList(indD,ind) = deltaList(1,ind);
            Dlist(indD,ind) = Dlist(1,ind);
        end
    end
end

%% figure
Qmin = min(Qlist(1,:));
deltaMin = min(deltaList(1,:));

f1Hndl = figure();
contourf(real(deltaList)/deltaMin, real(Qlist)/Qmin, log10(real(Dlist)/Dmin), 30);
ylabel('Q/Q_{min}');
xlabel('\delta/\delta_{min}');
c = colorbar();
c.Label.String = 'D/D_{min} [dB]';
grid on;
aHndl = gca;
set(aHndl, 'xscale', 'log');
set(aHndl, 'yscale', 'log');


f2Hndl = figure();
surf(real(Qlist)/Qmin, real(deltaList)/deltaMin, real(Dlist)/Dmin);
xlabel('Q/Q_{min}');
ylabel('\delta/\delta_{min}');
zlabel('D/D_{min}');
grid on;
aHndl = gca;
set(aHndl, 'xscale', 'log');
set(aHndl, 'yscale', 'log');
set(aHndl, 'zscale', 'log');

%% export
if direction == 0
    dirName = 'broadside';
else
    dirName = 'endfire';
end
save([dirName, '.mat'], 'Qlist', 'deltaList', 'Dlist', '-mat');

zMin = min(min(Dlist))/Dmin;
yMax = max(max(deltaList))/deltaMin;
xMax = max(max(Qlist))/Qmin;

Qlist(end+1,:) = nan;
deltaList(end+1,:) = nan;
Dlist(end+1,:) = nan;

data = [real(reshape(Qlist,(nParetoPoints(1)+1)*nParetoPoints(2),1))/Qmin,...
    real(reshape(deltaList,(nParetoPoints(1)+1)*nParetoPoints(2),1))/deltaMin,...
    real(reshape(Dlist,(nParetoPoints(1)+1)*nParetoPoints(2),1))/Dmin];

% surf
fileName = [dirName, '.tsv'];
save(fileName, 'data', '-ascii');

Str = fileread(fileName);
Str2 = strrep(Str, 'NaN', '');
FID = fopen(fileName, 'w');
fwrite(FID, Str2, 'uchar');
fclose(FID);

% xy projection
data0 = data(:,1);
data0(~isnan(data0)) = 0.7;
dataZ = [data(:,1:2), data0];

fileName = [dirName, 'Z.tsv'];
save(fileName, 'dataZ', '-ascii');

Str = fileread(fileName);
Str2 = strrep(Str, 'NaN', '');
FID = fopen(fileName, 'w');
fwrite(FID, Str2, 'uchar');
fclose(FID);

% xz projection
data0(~isnan(data0)) = 16;
dataY = [data(:,1), data0, data(:,3)];

fileName = [dirName, 'Y.tsv'];
save(fileName, 'dataY', '-ascii');

Str = fileread(fileName);
Str2 = strrep(Str, 'NaN', '');
FID = fopen(fileName, 'w');
fwrite(FID, Str2, 'uchar');
fclose(FID);

% yz projection
data0(~isnan(data0)) = 7;
dataX = [data0, data(:,2:3)];

fileName = [dirName, 'X.tsv'];
save(fileName, 'dataX', '-ascii');

Str = fileread(fileName);
Str2 = strrep(Str, 'NaN', '');
FID = fopen(fileName, 'w');
fwrite(FID, Str2, 'uchar');
fclose(FID);