%% exMinQvsDeltaVsGswpSelfRes
% This script provides computation of Q-factor, dissipation factor and gain
% pareto optimal points using convex cobination Q-delta and G sweep.
%
% AToM package Fundamental Bounds, info@antennatoolbox.com
% (c) 2020, Jakub Liska, CTU in Prague, jakub.liska@antennatoolbox.com
% mcode

clc, clear, close all;

nParetoPoints = [90, 20]; % [G-sweep, Q-delta]
verbosity = 1;

%% Mesh
Nx = 18;
Ny = 9;
PX = 2*ones(Ny, Nx);
[~, ~, mesh] = models.utilities.meshPublic.pixelGridToMesh(PX, 1);
%% MoM matrices
ka = 0.5; % electrical size
Zs = 0.01; % surface impedance

% smallest circumscribing sphere
a = models.utilities.meshPublic.getCircumsphere(mesh.nodes);

f = ka*models.utilities.constants.c0()/(2*pi*a); % frequaency
[OP, ~] = models.utilities.matrixOperators.MoM2D.batch.evaluate(mesh, f, Zs,...
   'normalize', true, 'symmetrize', true, 'quadOrder', 7,...
   'requests', {'R0', 'omW', 'Rmat', 'U', 'X0', 'Xmat'}, 'usegpu', false); % get matrix operators
% OP.U = OP.U_hndl(f, pi/2, 0, 'phi'); % endfire
% OP.U2 = OP.U_hndl(f, pi/2, pi, 'phi'); % endfire2
OP.U = OP.U_hndl(f, 0, 0, 'theta'); % broadside
OP.U2 = OP.U_hndl(f, pi, 0, 'theta'); % broadside

normMat_fun = @(M) M/norm(M);

normMat.R0 = normMat_fun(OP.R0);
normMat.W = normMat_fun(OP.omW);
normMat.X = normMat_fun(OP.X0 + OP.Xmat);
normMat.Rmat = normMat_fun(OP.Rmat);
normMat.U = normMat_fun(OP.U);
normMat.RL = normMat_fun(OP.R0 + OP.Rmat);

G_fun = @(x) 8*pi*real((x'*OP.U*x)/(x'*(OP.Rmat + OP.R0)*x));
G_fun2 = @(x) 8*pi*real((x'*OP.U2*x)/(x'*(OP.Rmat + OP.R0)*x));
Q_fun = @(x) 0.5*real(((x'*OP.omW*x) + abs(x'*OP.X0*x))/(x'*OP.R0*x));
delta_fun = @(x) real((x'*OP.Rmat*x)/(x'*OP.R0*x));

Q_constraintMat = @(Q0) normMat_fun(0.5*OP.omW - Q0*OP.R0);
D_constraintMat = @(D0) normMat_fun(8*pi*OP.U - D0*OP.R0);
delta_constraintMat = @(d0) normMat_fun(OP.Rmat - d0*OP.R0);

%% G pareto initialization
Qlist = nan(nParetoPoints);
deltaList = nan(nParetoPoints);
Glist = nan(nParetoPoints);

alphaList = [0, 0.5.^linspace(14,4,nParetoPoints(2)-2), 1]; %endfire 6:3

%% no G
lambdaVec = zeros(1,2);
for ind = 1:nParetoPoints(2)
   alpha = alphaList(ind);
   matP = normMat_fun(alpha*OP.omW + (1 - alpha)*OP.Rmat);
   
   [~, ~, Ivec, lambdaVec, dg] = solvers.QCQP.minAstBn( - normMat.R0,...
      lambdaVec, verbosity,...
      matP, {}, - 1,...
      normMat.X, {}, 0);
   
   if dg
      err = [Ivec'*matP*Ivec - 1;...
         Ivec'*normMat.X*Ivec];
      disp(err);
      %       if ind ~= 1
      %          Qlist(1,ind) = Qlist(1,ind-1);
      %          deltaList(1,ind) = deltaList(1,ind-1);
      %          Glist(1,ind) = Glist(1,ind-1);
      %       else
      %          Qlist(1,ind) = nan;
      %          deltaList(1,ind) = nan;
      %          Glist(1,ind) = nan;
      %          lambdaVec = zeros(1,2);
   end
   %    else
   Qlist(1,ind) = Q_fun(Ivec);
   deltaList(1,ind) = delta_fun(Ivec);
   Glist(1,ind) = max([G_fun(Ivec), G_fun2(Ivec)]);
   %    end
   
   
end

%% reduce to pareto
% lambdaVec = zeros(1,4);
% multList = nan(nParetoPoints(2),4);
% for ind = 1:nParetoPoints(2)
%    matQ = normMat_fun(Q_constraintMat(Qlist(1,ind)));
%    matDelta = normMat_fun(delta_constraintMat(deltaList(1,ind)));
%    
%    [~, ~, Ivec, lambdaVec, dg] = solvers.QCQP.minAstBn( - normMat.U,...
%       lambdaVec, verbosity,...
%       normMat.RL, {}, - 1,...
%       normMat.X, {}, 0,...
%       matQ, {}, 0,...
%       matDelta, {}, 0);
%    
%    if dg
%       err = [Ivec'*normMat.RL*Ivec - 1;...
%          Ivec'*normMat.X*Ivec;...
%          Ivec'*matQ*Ivec;...
%          Ivec'*matDelta*Ivec];
%       disp(err);
%       if ind ~= 1
%          Glist(1,ind) = Glist(1,ind-1);
%          multList(ind,:) = multList(ind-1,:);
%       else
%          Glist(1,ind) = nan;
%          multList(ind,:) = zeros(1,4);
%       end
%    else
%       Glist(1,ind) = G_fun(Ivec);
%       multList(ind,:) = lambdaVec;
%    end
%    
%    if ind > 1
%       lambdaVec = 2*multList(ind,:) - multList(ind-1,:); %prediction
%    end
% end

%% G
Gmin = min(Glist(1,:));

% max G
[Gmax, Ivec] = optimProblems.maxGselfRes(OP, 'verbosity', verbosity);
Qlist(end,:) = Q_fun(Ivec);
deltaList(end,:) = real((Ivec'*OP.Rmat*Ivec)/(Ivec'*OP.R0*Ivec));
Glist(end,:) = Gmax;

ec = 3;
Gswp = Gmax*linspace((Gmin/Gmax).^(1/ec), 1,...
   nParetoPoints(1) - 1).^ec;
Glist(2:end-1,:) = repmat(Gswp(1:end-1).', 1, nParetoPoints(2));

%% G sweep
for indG = 2:nParetoPoints(1)
   lambdaVec = zeros(1,3);
   for ind = 1:nParetoPoints(2)
      if Glist(indG,ind) > Glist(1,ind)
         alpha = alphaList(ind);
         
         matP = normMat_fun(alpha*OP.omW + (1 - alpha)*OP.Rmat);
         matG = normMat_fun(G_constraintMat(Glist(indG,ind)));
         
         try
            [~, ~, Ivec, lambdaVec, dg] = solvers.QCQP.minAstBn( - normMat.R0,...
               lambdaVec, verbosity,...
               matP, {}, - 1,...
               normMat.X, {}, 0,...
               matG, {}, 0);
         catch msg
            warning('Divergence of dual problem!');
            Qlist(indG,ind) = Qlist(indG-1,ind);
            deltaList(indG,ind) = deltaList(indG-1,ind);
            Glist(indG,ind) = Glist(indG-1,ind);
         end
         
         if dg
            err = [Ivec'*matP*Ivec - 1;...
               Ivec'*normMat.X*Ivec;...
               Ivec'*matG*Ivec];
            disp(err);
            
            Qlist(indG,ind) = Qlist(indG-1,ind);
            deltaList(indG,ind) = deltaList(indG-1,ind);
            Glist(indG,ind) = Glist(indG-1,ind);
         else
            Qlist(indG,ind) = Q_fun(Ivec);
            deltaList(indG,ind) = delta_fun(Ivec);
            Glist(indG,ind) = G_fun(Ivec);
         end
      else
         Qlist(indG,ind) = Qlist(1,ind);
         deltaList(indG,ind) = deltaList(1,ind);
         Glist(indG,ind) = Glist(1,ind);
      end
   end
end

%% figure
Qmin = min(Qlist(1,:));
deltaMin = min(deltaList(1,:));

figure();
surf(Qmin./real(Qlist), deltaMin./real(deltaList), real(Glist)/Gmax);
xlabel('Q_{min}/Q');
ylabel('\delta_{min}/\delta');
zlabel('G/G_{max}');

% figure();
% surf(real(Qlist), real(deltaList), real(Glist));
% xlabel('Q');
% ylabel('\delta');
% zlabel('G');

% save('QdeltaGparetoEndfire2Rec10to5.mat', 'Qlist', 'deltaList', 'Glist');